import { system, world } from "@minecraft/server";
import { clearDynamicProperty, getAllPlayersSafe, readDynamicProperty, SafetyCheckedPlayer, writeDynamicProperty } from "./utils/index";
const DYNAMIC_PROPERTY_ID = "achievements";
export const DEFAULT_ACHIEVEMENT_DATA = {
    defeat_50_opponents_consecutively: {
        value: 0,
        unlockValue: 50,
        skinIndex: 1,
        lang: "defeat_50_opponents_consecutively",
        completed: false,
    },
    meditate_near_warden: {
        value: 0,
        unlockValue: 1,
        skinIndex: 2,
        lang: "meditate_near_warden",
        completed: false,
    },
    kill_creeper: {
        value: 0,
        unlockValue: 1,
        skinIndex: 3,
        lang: "kill_creeper",
        completed: false,
    },
    birthday: {
        value: 0,
        unlockValue: 1,
        skinIndex: 4,
        lang: "birthday",
        completed: false,
    },
    block_attacks: {
        value: 0,
        unlockValue: 100,
        skinIndex: 5,
        lang: "block_attacks",
        completed: false,
    },
    defeat_opponents: {
        value: 0,
        unlockValue: 100,
        skinIndex: 6,
        lang: "defeat_opponents",
        completed: false,
    },
    activate_dragon_fury: {
        value: 0,
        unlockValue: 10,
        skinIndex: 7,
        lang: "activate_dragon_fury",
        completed: false,
    },
    complete_martial_arts: {
        value: 0,
        unlockValue: 4,
        skinIndex: 8,
        lang: "complete_martial_arts",
        completed: false,
    },
    max_out_skills: {
        value: 0,
        unlockValue: 5,
        skinIndex: 9,
        lang: "max_out_skills",
        completed: false,
    },
    defeat_50_vanilla_mobs: {
        value: 0,
        unlockValue: 50,
        skinIndex: 10,
        lang: "defeat_50_vanilla_mobs",
        completed: false,
    }
};
export function initializeAchievements(player) {
    if (readDynamicProperty(player, DYNAMIC_PROPERTY_ID) === undefined) {
        writeDynamicProperty(player, DYNAMIC_PROPERTY_ID, DEFAULT_ACHIEVEMENT_DATA);
    }
}
let date = new Date();
getAllPlayersSafe().forEach(player => {
    initializeAchievements(player);
    if (date.getDay() === 27 && date.getMonth() === 10) {
        incrementAchievement(new SafetyCheckedPlayer(player), "birthday");
    }
});
world.afterEvents.playerSpawn.subscribe(event => {
    if (event.initialSpawn) {
        initializeAchievements(new SafetyCheckedPlayer(event.player));
        if (date.getDay() === 27 && date.getMonth() === 10) {
            incrementAchievement(new SafetyCheckedPlayer(event.player), "birthday");
        }
    }
});
system.afterEvents.scriptEventReceive.subscribe(event => {
    if (event.sourceEntity && event.sourceEntity.typeId === "minecraft:player") {
        const player = new SafetyCheckedPlayer(event.sourceEntity);
        if (event.id === "ldz_240913:clear_achievements") {
            clearAchievements(player);
        }
        else if (event.id === "ldz_240913:grant_achievement") {
            if (event.message in DEFAULT_ACHIEVEMENT_DATA) {
                const achievement = DEFAULT_ACHIEVEMENT_DATA[event.message];
                setAchievementValue(player, event.message, achievement.unlockValue);
            }
        }
    }
});
export function checkAchievement(player, achievement) {
    let achievements = readDynamicProperty(player, DYNAMIC_PROPERTY_ID);
    if (achievements === undefined)
        return false;
    let achievementData = achievements[achievement];
    if (achievementData === undefined)
        return false;
    return achievementData.value >= achievementData.unlockValue || achievementData.completed;
}
export function getAchievementValue(player, achievement) {
    let achievements = readDynamicProperty(player, DYNAMIC_PROPERTY_ID);
    if (achievements === undefined)
        return -1;
    let achievementData = achievements[achievement];
    if (achievementData === undefined)
        return -1;
    return achievementData.value;
}
export function getAchievementUnlockValue(player, achievement) {
    let achievements = readDynamicProperty(player, DYNAMIC_PROPERTY_ID);
    if (achievements === undefined)
        return -1;
    let achievementData = achievements[achievement];
    if (achievementData === undefined)
        return -1;
    return achievementData.unlockValue;
}
export function getAchievements(player) {
    return readDynamicProperty(player, DYNAMIC_PROPERTY_ID) || {};
}
export function setAchievementValue(player, achievement, value) {
    let achievements = readDynamicProperty(player, DYNAMIC_PROPERTY_ID);
    if (achievements === undefined)
        return;
    achievements[achievement].value = value;
    writeDynamicProperty(player, DYNAMIC_PROPERTY_ID, achievements);
    checkAchievementCompletion(player, achievement);
}
export function setAchievementUnlockValue(player, achievement, value) {
    let achievements = readDynamicProperty(player, DYNAMIC_PROPERTY_ID);
    if (achievements === undefined)
        return;
    achievements[achievement].unlockValue = value;
    writeDynamicProperty(player, DYNAMIC_PROPERTY_ID, achievements);
}
export function setAchievements(player, achievements) {
    writeDynamicProperty(player, DYNAMIC_PROPERTY_ID, achievements);
}
// export function addAchievement(player: SafetyCheckedPlayer, achievement: Achievements, unlockValue: number): void {
//     let achievements = readDynamicProperty<IAchievements>(player, DYNAMIC_PROPERTY_ID);
//     if (achievements === undefined) return;
//     achievements[achievement] = {
//         value: 0,
//         unlockValue: unlockValue
//     };
//     writeDynamicProperty(player, DYNAMIC_PROPERTY_ID, achievements);
// }
export function incrementAchievement(player, achievement, amount = 1) {
    let achievements = readDynamicProperty(player, DYNAMIC_PROPERTY_ID);
    if (achievements === undefined)
        return;
    achievements[achievement].value += amount;
    writeDynamicProperty(player, DYNAMIC_PROPERTY_ID, achievements);
    checkAchievementCompletion(player, achievement);
}
export function isAchievementCompleted(player, achievement) {
    const achievements = readDynamicProperty(player, DYNAMIC_PROPERTY_ID);
    if (!achievements)
        return false;
    const thisAchievement = achievements[achievement];
    return thisAchievement.completed;
}
function checkAchievementCompletion(player, achievement) {
    const achievements = readDynamicProperty(player, DYNAMIC_PROPERTY_ID);
    if (!achievements)
        return;
    const thisAchievement = achievements[achievement];
    if (thisAchievement.completed)
        return;
    if (thisAchievement.value >= thisAchievement.unlockValue) {
        thisAchievement.completed = true;
        writeDynamicProperty(player, DYNAMIC_PROPERTY_ID, achievements);
        player.sendMessage({ translate: "ldz_240913:chat_message.achievement_unlocked" });
        player.playSound("ldz_240913:achievement_completed");
    }
}
export function getCompletedAchievements(player) {
    let achievements = readDynamicProperty(player, DYNAMIC_PROPERTY_ID);
    if (achievements === undefined)
        return [];
    let completedAchievements = [];
    for (let achievement in achievements) {
        if (achievements[achievement].value >= achievements[achievement].unlockValue) {
            completedAchievements.push(achievements[achievement]);
        }
        else if (achievements[achievement].completed) {
            completedAchievements.push(achievements[achievement]);
        }
    }
    return completedAchievements;
}
export function getAchievementsMenuBodyText(player) {
    var _a, _b;
    const achievements = readDynamicProperty(player, DYNAMIC_PROPERTY_ID);
    if (!achievements)
        return { rawtext: [{ text: "Error: failed to get achievement text. Reload." }] };
    let achievementText = { rawtext: [] };
    for (let achievementName in achievements) {
        const achievement = achievements[achievementName];
        if (achievement.value < achievement.unlockValue) {
            (_a = achievementText.rawtext) === null || _a === void 0 ? void 0 : _a.push({ translate: `ldz_240913:menu.achievements.${achievementName}.in_progress.title` }, { text: "\n" }, { translate: `ldz_240913:menu.achievements.${achievementName}.in_progress.status`, with: [String(achievement.unlockValue - achievement.value)] }, { text: "\n\n" });
        }
        else {
            (_b = achievementText.rawtext) === null || _b === void 0 ? void 0 : _b.push({ translate: `ldz_240913:menu.achievements.${achievementName}.completed.title` }, { text: "\n" }, { translate: `ldz_240913:menu.achievements.${achievementName}.completed.status`, with: [String(achievement.unlockValue)] }, { text: "\n\n" });
        }
    }
    return achievementText;
}
function clearAchievements(player) {
    clearDynamicProperty(player, DYNAMIC_PROPERTY_ID);
    initializeAchievements(player);
}
