import * as mc from "@minecraft/server";
import * as utils from "../utils/index";
export class SoundPlayer extends utils.ActorComponent {
    constructor() {
        super(...arguments);
        this.queuedSoundEvents = {};
    }
    playSound(eventId, dimension, location) {
        this.stopSounds();
        const soundEventPool = this.options[eventId];
        if (soundEventPool.length === 0) {
            utils.mcerror(`SoundEventPool for ${eventId} has no entries.`, "logSoundPlayer");
        }
        else if (soundEventPool.length === 1) {
            this.handleSoundEvent(soundEventPool[0], eventId, dimension, location);
        }
        else {
            // Disable sound randomization if there are any timed sound references in the event
            let doRandom = true;
            for (const soundEvent of soundEventPool) {
                if (typeof soundEvent === "object" && !("name" in soundEvent)) {
                    doRandom = false;
                    break;
                }
            }
            if (doRandom) {
                // Handle random sound events
                const soundEvent = soundEventPool[Math.floor(Math.random() * soundEventPool.length)];
                this.handleSoundEvent(soundEvent, eventId, dimension, location);
            }
            else {
                soundEventPool.forEach(soundEvent => this.handleSoundEvent(soundEvent, eventId, dimension, location));
            }
        }
    }
    stopSounds(eventId) {
        var _a;
        if (eventId) {
            const actorEventId = `${this.actor.id}:${eventId}`;
            utils.abortEvents(actorEventId);
            (_a = this.queuedSoundEvents[actorEventId]) === null || _a === void 0 ? void 0 : _a.forEach(soundReference => this.actor.dimension.runCommand(`stopsound @a[r=16] ${soundReference}`));
            delete this.queuedSoundEvents[actorEventId];
        }
        else {
            Object.entries(this.queuedSoundEvents).forEach(([eventId, soundReferences]) => {
                eventId = `${this.actor.id}:${eventId}`;
                utils.abortEvents(eventId);
                soundReferences.forEach(soundReference => {
                    let sound = typeof soundReference === "string" ? soundReference : soundReference.name;
                    utils.mclog(`Stopping sound: ${sound}`, "logSoundPlayer");
                    this.actor.dimension.runCommand(`stopsound @a[r=16] ${sound}`);
                });
            });
            this.queuedSoundEvents = {};
        }
    }
    handleSoundEvent(soundEvent, eventId, dimension = this.actor.dimension, location = this.actor.location) {
        var _a, _b;
        const actorEventId = `${this.actor.id}:${eventId}`;
        if (typeof soundEvent === "string" || "name" in soundEvent) {
            // Lone SoundReference
            utils.mclog(`Playing sound: ${soundEvent}`, "logSoundPlayer");
            this.queuedSoundEvents[actorEventId] ? (_a = this.queuedSoundEvents[actorEventId]) === null || _a === void 0 ? void 0 : _a.push(soundEvent) : this.queuedSoundEvents[actorEventId] = [soundEvent];
            this.playSoundToPlayers(soundEvent, dimension, location);
        }
        else {
            // TimedSoundEvent
            for (const [delay, soundReference] of Object.entries(soundEvent)) {
                const delayTicks = Number(delay) * mc.TicksPerSecond;
                let sounds = (typeof soundReference === "string" || "name" in soundReference) ? [soundReference] : soundReference;
                let sound = sounds[Math.floor(Math.random() * sounds.length)];
                utils.mclog(`Playing sound: [${delayTicks}] ${sound}`, "logSoundPlayer");
                (_b = this.queuedSoundEvents[actorEventId]) === null || _b === void 0 ? void 0 : _b.push(sound);
                utils.waitForEvent(delayTicks, () => this.playSoundToPlayers(sound, dimension, location));
            }
        }
    }
}
