import { EntityDamageCause, system, world } from "@minecraft/server";
import { Actor, ActorComponent, clamp, getEntitiesSafe, getRandomInteger, mclog, Projectile, readDynamicProperty, SafetyCheckedEntity, SafetyCheckedPlayer, waitForEvent, waitForTicks } from "../utils/index";
import { BruceLee } from "./bruce_lee";
import { UNARMED_DEFAULT_ATTACKS } from "./unarmed_attacks";
import { incrementAchievement, isAchievementCompleted } from "../achievements";
const BOSSES = [
    "minecraft:wither",
    "minecraft:ender_dragon",
];
const BOSS_DMG_MULTIPLIER = 2;
class HomingProjectile extends Projectile {
    updateDirection(newDirection) {
        this.Direction = newDirection;
    }
}
export class ActorAttack extends ActorComponent {
    constructor(actor, options = UNARMED_DEFAULT_ATTACKS) {
        super(actor, options);
        this.actor = actor;
        this.options = options;
        this.eventBinding = this.onEventRecieved.bind(this);
        this.validTargets = [
            { type: "player" },
            { families: ["mob"] },
            { families: ["ldz_240913:opponent"] }
        ];
        this.nextHitTime = 0;
    }
    onComponentAdded() {
        system.afterEvents.scriptEventReceive.subscribe(this.eventBinding);
    }
    onComponentRemoved() {
        system.afterEvents.scriptEventReceive.unsubscribe(this.eventBinding);
    }
    addAttack(attack) {
        this.options.attacks.push(attack);
    }
    addFuryAttack(attack) {
        this.options.furyAttacks.push(attack);
    }
    removeAttack(attack) {
        const index = this.options.attacks.indexOf(attack);
        if (index !== -1) {
            this.options.attacks.splice(index, 1);
        }
    }
    removeFuryAttack(attack) {
        const index = this.options.furyAttacks.indexOf(attack);
        if (index !== -1) {
            this.options.furyAttacks.splice(index, 1);
        }
    }
    furyMode() {
        var _a;
        if (!this.actor.Properties.CanAttack)
            return;
        this.actor.Properties.CanAttack = false;
        this.actor.Properties.PerformingDragonFury = true;
        const attackIndex = this.actor.Properties.DragonFuryLevel;
        const attack = this.options.furyAttacks[attackIndex - 1];
        let victims = [];
        this.validTargets.forEach(target => {
            victims = victims.concat(getEntitiesSafe(this.actor.dimension, Object.assign({ location: this.actor.location, maxDistance: attack.range }, target)));
        });
        this.actor.IdleLines.stopTimer();
        this.actor.addEffect("slowness", attack.duration.endTime * 20, { amplifier: 255, showParticles: false });
        this.actor.playAnimation(`animation.ldz_240913.bruce_lee.start_fury_attack_${attackIndex}`, {
            "controller": "animation.ldz_240913.bruce_lee.attack_controller"
        });
        (_a = this.actor.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound(attack.sound);
        if (victims.length > 0) {
            victims.forEach(victim => {
                attack.duration.damageTimes.forEach(time => {
                    waitForEvent(time * 20, () => {
                        var _a, _b;
                        if (this.actor.isValid() && this.actor.Properties.Alive && victim.isValid() && victim.id !== this.actor.Properties.OwnerId) {
                            if (readDynamicProperty(world, "ldz_240913:fury_hits_players") === false && victim.typeId === "minecraft:player")
                                return;
                            victim.applyDamage(attack.damage + this.actor.Properties.AttackDamage, { cause: EntityDamageCause.entityAttack, damagingEntity: this.actor });
                            if (BOSSES.includes(victim.typeId)) {
                                victim.applyDamage((attack.damage + this.actor.Properties.AttackDamage) * BOSS_DMG_MULTIPLIER, { cause: EntityDamageCause.blockExplosion, damagingEntity: this.actor });
                            }
                            this.actor.Properties.DamageDealt += attack.damage;
                            mclog(`Dealt ${attack.damage} damage to ${victim.typeId}`, "BruceLee");
                            if (((_b = (_a = victim.Health) === null || _a === void 0 ? void 0 : _a.currentValue) !== null && _b !== void 0 ? _b : 0) <= 0 && victim.isValid() && !victim.hasTag("ldz_240913:killed_by_bruce_lee")) {
                                if (victim.matches({ families: ["ldz_240913:opponent"] })) {
                                    this.actor.Properties.OpponentsDefeated += 1;
                                    if (this.actor.Owner && !isAchievementCompleted(this.actor.Owner, "defeat_50_opponents_consecutively")) {
                                        mclog("Increment defeat_50_opponents_consecutively achievement");
                                        incrementAchievement(new SafetyCheckedPlayer(this.actor.Owner), "defeat_50_opponents_consecutively");
                                    }
                                    this.actor.gainExperience(2);
                                }
                                else {
                                    this.actor.Properties.MobsDefeated += 1;
                                    if (victim.typeId === "minecraft:creeper" && this.actor.Owner) {
                                        incrementAchievement(this.actor.Owner, "kill_creeper");
                                    }
                                    this.actor.gainExperience(1);
                                }
                                victim.addTag("ldz_240913:killed_by_bruce_lee");
                            }
                        }
                    });
                });
            });
        }
        waitForEvent(attack.duration.endTime * 20, () => {
            if (this.actor.isValid()) {
                this.actor.playAnimation(`animation.ldz_240913.bruce_lee.stop_fury_attack`, {
                    "controller": "animation.ldz_240913.bruce_lee.attack_controller"
                });
                this.actor.Properties.CanAttack = true;
                this.actor.Properties.PerformingDragonFury = false;
                this.actor.IdleLines.startTimer();
                waitForEvent(1, () => {
                    if (this.actor.isValid()) {
                        this.actor.playAnimation(`animation.ldz_240913.bruce_lee.stop_fury_attack`, {
                            "controller": "animation.ldz_240913.bruce_lee.attack_controller"
                        });
                    }
                });
            }
        });
    }
    onEventRecieved(event) {
        var _a;
        if (this.nextHitTime >= system.currentTick)
            return;
        if (!this.actor.Properties.CanAttack)
            return;
        if (event.id === "ldz_240913:attack" && event.message === "attacker" && ((_a = event.sourceEntity) === null || _a === void 0 ? void 0 : _a.id) === this.actor.id) {
            const subEvent = (subEvent) => {
                if (subEvent.id === "ldz_240913:attack" && subEvent.message === "victim") {
                    const victim = subEvent.sourceEntity;
                    if ((victim === null || victim === void 0 ? void 0 : victim.typeId) === "ldz_240913:bruce_distraction")
                        return;
                    if (victim && victim.id !== this.actor.id && !BOSSES.includes(victim.typeId)) {
                        mclog(`Regular attack on victim ${victim.typeId}`, 'BruceLee');
                        this.performAttack(new SafetyCheckedEntity(victim));
                    }
                    else if (victim && victim.id !== this.actor.id && BOSSES.includes(victim.typeId)) {
                        mclog('Boss attack', 'BruceLee');
                        this.performBossAttack(new SafetyCheckedEntity(victim));
                    }
                }
            };
            system.afterEvents.scriptEventReceive.subscribe(subEvent);
            waitForEvent(1, () => {
                system.afterEvents.scriptEventReceive.unsubscribe(subEvent);
            });
        }
    }
    performBossAttack(victim) {
        var _a;
        if (!this.actor.Properties.CanAttack && !this.actor.Properties.Meditating)
            return;
        const attackIndex = getRandomInteger(0, this.options.bossAttacks.length - 1);
        const attack = this.options.bossAttacks[attackIndex];
        this.nextHitTime = system.currentTick + (attack.duration.endTime * 20) + (attack.cooldown * 20);
        this.actor.addEffect("slowness", attack.duration.endTime * 20, { amplifier: 255, showParticles: false });
        this.actor.playAnimation(`animation.ldz_240913.bruce_lee.${attack.animation}`, {
            "controller": "animation.ldz_240913.bruce_lee.attack_controller"
        });
        waitForTicks(1).then(() => {
            this.actor.playAnimation(`animation.ldz_240913.bruce_lee.${attack.animation}`, {
                "controller": "animation.ldz_240913.bruce_lee.attack_controller"
            });
        });
        let damage = attack.damage + this.actor.Properties.AttackDamage;
        this.actor.IdleLines.stopTimer();
        (_a = this.actor.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound(attack.sound);
        attack.duration.damageTimes.forEach((time, idx) => {
            waitForEvent(time * 20, async () => {
                let proj = new HomingProjectile(this.actor.getHeadLocation(), this.actor.dimension, "ldz_240913:dragon_projectile", victim.location.add({ x: 0, y: 1, z: 0 }).subtract(this.actor.getHeadLocation()).normalize().multiply(0.5), this.actor, {
                    faceDirection: true,
                    gravity: 0,
                    hit_sound: "",
                    power: 1.8,
                    on_hit: {
                        damage: damage,
                        destroy_on_hit: false,
                        multiple_targets: true,
                        knockback: false,
                        async hit_callback(projectile, hitEntities) {
                            let actor = Actor.getActor(projectile.Owner.id, BruceLee);
                            if (actor) {
                                hitEntities.forEach(async (entity) => {
                                    if (entity.id === victim.id) {
                                        entity.applyDamage(damage, { cause: EntityDamageCause.blockExplosion, damagingEntity: actor });
                                        onHit(actor, victim, attack, idx);
                                        entity.dimension.spawnParticle("ldz_240913:dragon_spiral_impact", entity.location.add({ x: 0, y: 1, z: 0 }));
                                        entity.dimension.spawnParticle("ldz_240913:dragon_flames", entity.location.add({ x: 0, y: 1, z: 0 }));
                                        entity.dimension.playSound("ldz_240913:bruce_lee.dragon_projectile_impact", entity.location);
                                        await waitForTicks(1);
                                        proj.Entity.remove();
                                    }
                                });
                            }
                        },
                    }
                });
                while (proj.Entity.isValid()) {
                    proj.updateDirection(victim.location.add({ x: 0, y: 1, z: 0 }).subtract(this.actor.getHeadLocation()).normalize().multiply(0.5));
                    await waitForTicks(1);
                }
            });
        });
        waitForEvent(attack.duration.endTime * 20, () => {
            if (this.actor.isValid()) {
                mclog(` [${system.currentTick / 20}] Stopping animation animation.ldz_240913.bruce_lee.stop_attack`, "BruceLee");
                this.actor.playAnimation(`animation.ldz_240913.bruce_lee.stop_boss_attack`, {
                    "controller": "animation.ldz_240913.bruce_lee.attack_controller"
                });
                this.actor.IdleLines.startTimer();
                waitForEvent(1, () => {
                    if (this.actor.isValid()) {
                        mclog(` [${system.currentTick / 20}] Stopping animation animation.ldz_240913.bruce_lee.stop_attack`, "BruceLee");
                        this.actor.playAnimation(`animation.ldz_240913.bruce_lee.stop_boss_attack`, {
                            "controller": "animation.ldz_240913.bruce_lee.attack_controller"
                        });
                    }
                });
            }
        });
    }
    performAttack(victim) {
        var _a;
        if (!this.actor.Properties.CanAttack)
            return;
        const attackIndex = getRandomInteger(0, clamp(this.options.attacks.length - 1, 0, 3 + (Math.floor(this.actor.Properties.Level / 10)) * 2));
        const attack = this.options.attacks[attackIndex];
        const distance = this.actor.location.distance(victim.location);
        if (distance <= attack.range) {
            this.nextHitTime = system.currentTick + (attack.duration.endTime * 20) + (attack.cooldown * 20);
            this.actor.addEffect("slowness", attack.duration.endTime * 20, { amplifier: 255, showParticles: false });
            mclog(` [${system.currentTick / 20}] Playing animation animation.${this.actor.typeId.replace(":", ".")}.${attack.animation}`, "BruceLee");
            this.actor.playAnimation(`animation.ldz_240913.bruce_lee.${attack.animation}`, {
                "controller": "animation.ldz_240913.bruce_lee.attack_controller"
            });
            waitForEvent(1, () => {
                if (this.actor.isValid()) {
                    this.actor.IdleLines.stopTimer();
                }
            });
            (_a = this.actor.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound(attack.sound);
            attack.duration.damageTimes.forEach((time, idx) => {
                var _a;
                let victims = [victim];
                if ((_a = attack.duration.canMultiHit) === null || _a === void 0 ? void 0 : _a[idx]) {
                    this.validTargets.forEach(target => {
                        victims = victims.concat(getEntitiesSafe(this.actor.dimension, Object.assign({ location: this.actor.location, maxDistance: attack.range }, target)));
                    });
                }
                waitForEvent(time * 20, () => {
                    victims.forEach(victim => {
                        if (this.actor.isValid() && this.actor.Properties.Alive && victim.isValid()) {
                            onHit(this.actor, victim, attack, idx);
                        }
                    });
                });
            });
            waitForEvent(attack.duration.endTime * 20, () => {
                if (this.actor.isValid()) {
                    mclog(` [${system.currentTick / 20}] Stopping animation animation.ldz_240913.bruce_lee.stop_attack`, "BruceLee");
                    this.actor.playAnimation(`animation.ldz_240913.bruce_lee.stop_attack`, {
                        "controller": "animation.ldz_240913.bruce_lee.attack_controller"
                    });
                    this.actor.IdleLines.startTimer();
                    waitForEvent(1, () => {
                        if (this.actor.isValid()) {
                            mclog(` [${system.currentTick / 20}] Stopping animation animation.ldz_240913.bruce_lee.stop_attack`, "BruceLee");
                            this.actor.playAnimation(`animation.ldz_240913.bruce_lee.stop_attack`, {
                                "controller": "animation.ldz_240913.bruce_lee.attack_controller"
                            });
                        }
                    });
                }
            });
        }
    }
}
function onHit(actor, victim, attack, damageInstance) {
    var _a, _b;
    victim.applyDamage((attack.damage + actor.Properties.AttackDamage) * BOSS_DMG_MULTIPLIER, { cause: EntityDamageCause.entityAttack, damagingEntity: actor });
    actor.Properties.DamageDealt += attack.damage;
    mclog(`Dealt ${attack.damage} damage to ${victim.typeId}`, "BruceLee");
    // Apply our own custom knockback amount
    if (victim.typeId !== "minecraft:player")
        victim.clearVelocity();
    if (!attack.duration.clearVelocity || attack.duration.clearVelocity[damageInstance]) {
        victim.applyKnockback(actor.getRotation().x, actor.getRotation().y, 0.4, 0.2);
    }
    // Perform actions if victim is dead
    if (((_b = (_a = victim.Health) === null || _a === void 0 ? void 0 : _a.currentValue) !== null && _b !== void 0 ? _b : 0) <= 0 && victim.isValid() && !victim.hasTag("ldz_240913:killed_by_bruce_lee")) {
        if (victim.matches({ families: ["ldz_240913:opponent"] })) {
            actor.Properties.OpponentsDefeated += 1;
            if (actor.Owner && !isAchievementCompleted(actor.Owner, "defeat_50_opponents_consecutively")) {
                mclog("Increment defeat_50_opponents_consecutively achievement");
                incrementAchievement(new SafetyCheckedPlayer(actor.Owner), "defeat_50_opponents_consecutively");
            }
            actor.gainExperience(2);
        }
        else {
            actor.Properties.MobsDefeated += 1;
            if (victim.typeId === "minecraft:creeper" && actor.Owner) {
                incrementAchievement(actor.Owner, "kill_creeper");
            }
            else if (victim.typeId.includes("minecraft:") && actor.Owner) {
                incrementAchievement(actor.Owner, "defeat_50_vanilla_mobs");
            }
            actor.gainExperience(1);
        }
        victim.addTag("ldz_240913:killed_by_bruce_lee");
    }
}
