var _a, _b;
var _c;
import * as mc from "@minecraft/server";
import * as utils from "../utils/index";
import { BruceLee } from "./bruce_lee";
import { mclog } from "../utils/index";
export class BackupSystem {
    static get BackupRegistry() {
        return Object.assign({}, this.backupRegistry);
    }
    static PropertyRegistry(id) {
        var _a;
        return Object.assign({}, (_a = utils.readDynamicProperty(mc.world, `propertyRegistry.${id}`)) !== null && _a !== void 0 ? _a : {});
    }
    static registerLoadEvent(category, callback) {
        this.onLoadEvents.set(category, callback);
    }
    static registerBackup(id, category, categoryLimit, propertyRegistry) {
        this.backupRegistry[id] = category;
        let amount = Object.keys(this.backupRegistry).filter(key => this.backupRegistry[key] === category).length;
        if (amount > categoryLimit && categoryLimit > 0) {
            utils.mclog(`${category} limit reached, removing oldest backup`, this.name);
            let oldest = Object.keys(this.backupRegistry).filter(key => this.backupRegistry[key] === category)[0];
            if (oldest) {
                this.removeBackup(oldest);
            }
        }
        utils.writeDynamicProperty(mc.world, "backupRegistry", this.backupRegistry);
        utils.writeDynamicProperty(mc.world, `propertyRegistry.${id}`, propertyRegistry);
        utils.mclog(`Adding ${category} at ${id} to:\n` + JSON.stringify(this.backupRegistry, null, "\t"), this.name);
    }
    static removeBackup(id) {
        delete this.backupRegistry[id];
        utils.writeDynamicProperty(mc.world, "backupRegistry", this.backupRegistry);
        utils.clearDynamicProperty(mc.world, `propertyRegistry.${id}`);
        mc.world.structureManager.delete("mystructure:" + id);
        utils.mclog(`Removing ${id} from:\n` + JSON.stringify(this.backupRegistry, null, "\t"), this.name);
    }
    static registerDelete(id, category) {
        const entity = utils.getEntitySafe(id);
        if (entity === null || entity === void 0 ? void 0 : entity.isValid()) {
            utils.mclog(`Found entity for deletion: ${id}, removing`, this.name);
            try {
                entity.remove();
                return;
            }
            catch (error) { }
        }
        utils.mclog(`Could not find entity for ${id}, adding to delete registry`, this.name);
        this.deleteRegistry[id] = category;
        utils.writeDynamicProperty(mc.world, "deleteRegistry", this.deleteRegistry);
    }
    static removeDelete(id) {
        delete this.deleteRegistry[id];
        utils.writeDynamicProperty(mc.world, "deleteRegistry", this.deleteRegistry);
    }
    static loadFromBackup(id, dimension, location, remove = true) {
        const category = this.backupRegistry[id];
        if (!category)
            return Promise.resolve(undefined);
        utils.mclog(`Loading backup ${id} to ${category}`, this.name);
        const structureName = "mystructure:" + id;
        const structure = mc.world.structureManager.get(structureName);
        if (!(structure === null || structure === void 0 ? void 0 : structure.isValid()))
            return Promise.resolve(undefined);
        mc.world.structureManager.place(structure, dimension, location);
        const entities = utils.getEntitiesSafe(dimension, { location, maxDistance: 10 });
        const newEntity = entities.find(entity => utils.readDynamicProperty(entity, "structure_id") === id);
        entities.forEach(entity => {
            if (entity.isValid() && entity.hasTag("extra_structure_entity")) {
                entity.remove();
            }
        });
        return new Promise(resolve => {
            if (newEntity) {
                utils.waitForEvent(1, () => {
                    var _a;
                    const actor = utils.Actor.getActor(newEntity.id, BruceLee);
                    if (actor) {
                        utils.mclog(`Loaded backup ${id} to ${actor.ActorId}:${actor.id}`, this.name);
                        if (remove) {
                            (_a = this.onLoadEvents.get(category)) === null || _a === void 0 ? void 0 : _a(id, actor.id);
                            this.registerDelete(id, category);
                            this.removeBackup(id);
                        }
                        resolve(actor);
                    }
                    else {
                        resolve(undefined);
                    }
                });
            }
            else {
                utils.mcwarn(`Could not find entity for backup ${id}`, this.name);
                resolve(undefined);
            }
        });
    }
    static clearBackups() {
        utils.clearDynamicProperty(mc.world, "backupRegistry");
        utils.clearDynamicProperty(mc.world, "deleteRegistry");
        utils.clearDynamicProperty(mc.world, "propertyRegistry");
    }
}
_c = BackupSystem;
BackupSystem.backupRegistry = (_a = utils.readDynamicProperty(mc.world, "backupRegistry")) !== null && _a !== void 0 ? _a : {};
BackupSystem.deleteRegistry = (_b = utils.readDynamicProperty(mc.world, "deleteRegistry")) !== null && _b !== void 0 ? _b : {};
BackupSystem.onLoadEvents = new Map();
BackupSystem.event = mc.world.afterEvents.entityLoad.subscribe(event => {
    const category = _c.deleteRegistry[event.entity.id];
    if (category) {
        utils.mclog(`Entity marked for deletion found: ${event.entity.id}, removing`, _c.name);
        event.entity.remove();
        _c.removeDelete(event.entity.id);
    }
});
export class BackupComponent extends utils.ActorComponent {
    constructor() {
        super(...arguments);
        this.saveBinding = this.save.bind(this);
        this.onKilledBinding = this.onKilled.bind(this);
    }
    onComponentAdded() {
        this.options.backupEvents.forEach(event => {
            this.actor.subscribe(event, this.saveBinding);
        });
        if (this.options.deleteBackupOnKilled) {
            mc.world.afterEvents.entityDie.subscribe(this.onKilledBinding, { entities: [this.actor.UnsafeEntity] });
        }
        // this.save();
    }
    onComponentRemoved() {
        this.options.backupEvents.forEach(event => {
            this.actor.unsubscribe(event, this.saveBinding);
        });
        mc.world.afterEvents.entityDie.unsubscribe(this.onKilledBinding);
    }
    onKilled(event) {
        if (this.options.deleteBackupOnKilled) {
            BackupSystem.removeBackup(this.actor.id);
        }
    }
    save() {
        var _a, _b, _d;
        const nearbyEntities = utils.getEntitiesSafe(this.actor.dimension, { location: this.actor.location, maxDistance: 10 });
        nearbyEntities.forEach(entity => {
            if (entity.id !== this.actor.id) {
                entity.addTag("extra_structure_entity");
            }
        });
        mclog(`Saving ${this.actor.ActorId}:${this.actor.id} to backup ${(_a = this.options.saveCategory) !== null && _a !== void 0 ? _a : "default"}`, this.id);
        utils.writeDynamicProperty(this.actor, "structure_id", this.actor.id);
        BackupSystem.registerBackup(this.actor.id, (_b = this.options.saveCategory) !== null && _b !== void 0 ? _b : "default", this.options.categoryLimit, {
            Experience: this.actor.Properties.Experience,
            ExperienceToNextLevel: this.actor.Properties.ExperienceToNextLevel,
            Level: this.actor.Properties.Level,
            SkillPoints: this.actor.Properties.SkillPoints,
            MaxHealth: this.actor.Properties.MaxHealth,
            MartialArt: this.actor.Properties.MartialArt,
            BlockChance: this.actor.Properties.BlockChance,
            DragonFuryLevel: this.actor.Properties.DragonFuryLevel,
            FightingPriority: this.actor.Properties.FightingPriority,
            Mood: this.actor.Properties.Mood,
            Outfit: this.actor.Properties.Outfit,
            Hairstyle: this.actor.Properties.Hairstyle,
            BlockedDamage: this.actor.Properties.BlockedDamage,
            HealthFlash: this.actor.Properties.HealthFlash,
            OwnerId: this.actor.Properties.OwnerId,
            AttackDamage: this.actor.Properties.AttackDamage,
            Speed: this.actor.Properties.Speed,
            DamageTaken: this.actor.Properties.DamageTaken,
            DamageDealt: this.actor.Properties.DamageDealt,
            AttacksBlocked: this.actor.Properties.AttacksBlocked,
            TimesKnockedOut: this.actor.Properties.TimesKnockedOut,
            TimeMeditating: this.actor.Properties.TimeMeditating,
            MobsDefeated: this.actor.Properties.MobsDefeated,
            OpponentsDefeated: this.actor.Properties.OpponentsDefeated,
            Retired: this.actor.Properties.Retired
        });
        mc.world.structureManager.delete("mystructure:" + this.actor.id);
        mc.world.structureManager.createFromWorld("mystructure:" + this.actor.id, this.actor.dimension, this.actor.location, this.actor.location, {
            includeBlocks: false,
            includeEntities: true,
            saveMode: mc.StructureSaveMode.World,
        });
        nearbyEntities.forEach(entity => entity.removeTag("extra_structure_entity"));
        utils.mclog(`Saving ${this.actor.ActorId}:${this.actor.id} to backup ${(_d = this.options.saveCategory) !== null && _d !== void 0 ? _d : "default"}`, this.id);
    }
    load(dimension, location) {
        return BackupSystem.loadFromBackup(this.actor.id, dimension, location);
    }
    static load(id, dimension, location) {
        return BackupSystem.loadFromBackup(id, dimension, location);
    }
}
