import * as mc from "@minecraft/server";
import * as utils from "../utils/index";
import { BruceLeeProperties } from "./properties";
import { showMenuToPlayer } from "./menu";
import { ActorAttack } from "./attack";
import { TABAK_TOYOK_DEFAULT_ATTACKS } from "./tabak_toyok_attacks";
import { BO_STAFF_DEFAULT_ATTACKS } from "./bo_staff_attacks";
import { ESCRIMA_STICKS_DEFAULT_ATTACKS } from "./escrima_sticks_attacks";
import { incrementAchievement, isAchievementCompleted, setAchievementValue } from "../achievements";
import { PersistantActorStorage } from "./storage";
import { BRUCE_LEE_SOUNDS, BruceLeeSoundPlayer } from "./sound_player";
import { BruceLeeIdleLinesComponent } from "./idle_lines";
import { BackupComponent, BackupSystem } from "./backup";
import { readDynamicProperty } from "../utils/index";
const LOGTAG = "BruceLee";
const HEALTH_PER_DRAGON_CRYSTAL_SHARD = 12;
const BLOCK_DURATION = 2;
const DEAD_MOOD = 7;
export const BRUCE_LEE_OUTFITS = [
    "big_boss_sash",
    "dark_blue_wing_chun",
    "brown_tai_chi",
    "dark_green",
    "formal_suit_black",
    "red_tracksuit",
    "tank_top",
    "white_shirt_blue_pants",
    "formal_suit_white",
    "yellow_jumpsuit",
    "shirtless",
];
export class BruceLee extends utils.Actor {
    constructor(entity) {
        super(entity);
        this.onHurtBinding = this.onHurt.bind(this);
        this.scriptEventBinding = this.scriptEventHandler.bind(this);
        mc.world.afterEvents.entityHurt.subscribe(this.onHurtBinding, {
            entityTypes: ["ldz_240913:bruce_lee"]
        });
        mc.system.afterEvents.scriptEventReceive.subscribe(this.scriptEventBinding);
        entity.addTag("ldz_240913:distance_20");
    }
    get Properties() {
        return this.getActorComponent(BruceLeeProperties);
    }
    get SoundPlayer() {
        return this.getActorComponent(BruceLeeSoundPlayer);
    }
    get IdleLines() {
        return this.getActorComponent(BruceLeeIdleLinesComponent);
    }
    onInteract(player) {
        var _a, _b, _c;
        if (this.Properties.Retired)
            return;
        utils.mclog(`Bruce Lee interacted by ${player.name}`, LOGTAG);
        if (this.Properties.OwnerId === player.id && this.Properties.Alive) {
            //Interact with guide book
            if (((_a = player.MainHand) === null || _a === void 0 ? void 0 : _a.typeId) === "ldz_240913:guide_book") {
                this.onGuideBookInteract(player);
                //Interact with dragon crystal shard
            }
            else if (((_b = player.MainHand) === null || _b === void 0 ? void 0 : _b.typeId) === "ldz_240913:dragon_crystal_shard") {
                this.onInteractWithDragonCrystal(player);
                //Interact with scroll
            }
            else if (((_c = player.MainHand) === null || _c === void 0 ? void 0 : _c.typeId) === "ldz_240913:scroll" && this.Properties.Level < 40) {
                this.onInteractWithScroll(player);
                //Interact while meditating
            }
            else if (this.Properties.Meditating) {
                this.Properties.Meditating = false;
                this.removeEffect("slowness");
                //Interact with anything else
            }
            else {
                showMenuToPlayer(this, player);
            }
        }
    }
    onInteractWithDragonCrystal(player) {
        var _a, _b;
        if (((_a = player.MainHand) === null || _a === void 0 ? void 0 : _a.typeId) === "ldz_240913:dragon_crystal_shard" && this.Properties.Health < this.Properties.MaxHealth) {
            this.Properties.Health += HEALTH_PER_DRAGON_CRYSTAL_SHARD;
            if (player.MainHand.amount > 1 && player.getGameMode() !== "creative") {
                let item = player.MainHand;
                item.amount -= 1;
                player.MainHand = item;
            }
            else if (player.getGameMode() !== "creative") {
                player.MainHand = undefined;
            }
            this.dimension.spawnParticle("ldz_240913:bruce_lee_heal", this.location);
            (_b = this.SoundPlayer) === null || _b === void 0 ? void 0 : _b.playSound("heal");
        }
        else {
            showMenuToPlayer(this, player);
        }
    }
    onInteractWithScroll(player) {
        var _a, _b;
        if (this.Properties.Level < 40 && !(this.Properties.MartialArt === "none" && this.Properties.Level >= 10)) {
            this.gainExperience(5 + (this.Properties.Level * 5));
            if (((_a = player.MainHand) === null || _a === void 0 ? void 0 : _a.amount) && player.MainHand.amount > 1 && player.getGameMode() !== "creative") {
                let item = player.MainHand;
                item.amount -= 1;
                player.MainHand = item;
            }
            else if (player.getGameMode() !== "creative") {
                player.MainHand = undefined;
            }
            this.dimension.spawnParticle("ldz_240913:bruce_lee_heal", this.location);
            (_b = this.SoundPlayer) === null || _b === void 0 ? void 0 : _b.playSound("heal");
        }
        else {
            showMenuToPlayer(this, player);
        }
    }
    scriptEventHandler(event) {
        var _a;
        if (!event.sourceEntity || event.sourceEntity.id !== this.id)
            return;
        if (event.id === "ldz_240913:bruce_lee.meditate") {
            this.onMeditate();
        }
        else if (event.id === "ldz_240913:bruce_lee.charge_dragon_fury") {
            this.chargeDragonFury();
        }
        else if (event.id === "ldz_240913:bruce_lee.block") {
            this.onBlock();
        }
        else if (event.id === "ldz_240913:bruce_lee.meditating_near_warden") {
            if (this.Owner) {
                setAchievementValue(this.Owner, "meditate_near_warden", 1);
            }
        }
        else if (event.id === "ldz_240913:test_sound") {
            (_a = this.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound("heal");
        }
    }
    onGuideBookInteract(player) {
        var _a;
        if (this.Properties.DragonFuryCharge === 100) {
            this.Properties.DragonFuryCharge = 0;
            incrementAchievement(player, "activate_dragon_fury");
            (_a = this.getActorComponent(ActorAttack)) === null || _a === void 0 ? void 0 : _a.furyMode();
        }
        else if (player.location.distance(this.location) < 5) {
            showMenuToPlayer(this, player);
        }
    }
    toggleOutline(state, player) {
        if (state) {
            this.playAnimation("animation.ldz_240913.bruce_lee.start_outline", {
                players: [player.nameTag]
            });
        }
        else {
            this.playAnimation("animation.ldz_240913.bruce_lee.stop_outline", {
                players: [player.nameTag]
            });
        }
    }
    onPropertyChange(property, value) {
        var _a;
        (_a = this.getActorComponent(PersistantActorStorage)) === null || _a === void 0 ? void 0 : _a.onPropertyChange(property, value);
    }
    onConstructed() {
        var _a;
        this.addActorComponent(utils.InteractComponent);
        this.addActorComponent(BruceLeeProperties);
        this.addActorComponent(BruceLeeSoundPlayer, BRUCE_LEE_SOUNDS);
        this.addActorComponent(BruceLeeIdleLinesComponent);
        if (this.Properties.Health === -1) {
            this.Properties.Health = this.Properties.MaxHealth;
        }
        this.Properties.CanAttack = true;
        this.triggerEvent("revive");
        this.Properties.PerformingDragonFury = false;
        this.playAnimation(`animation.ldz_240913.bruce_lee.stop_attack`);
        switch (this.Properties.MartialArt) {
            case "tabak_toyok":
                this.addActorComponent(ActorAttack, TABAK_TOYOK_DEFAULT_ATTACKS);
                break;
            case "bo_staff":
                this.addActorComponent(ActorAttack, BO_STAFF_DEFAULT_ATTACKS);
                break;
            case "escrima_sticks":
                this.addActorComponent(ActorAttack, ESCRIMA_STICKS_DEFAULT_ATTACKS);
                break;
            default:
                this.addActorComponent(ActorAttack);
                break;
        }
        utils.getEntitiesSafe(this.dimension, { tags: [`ldz_240913:${this.id}`] }).forEach((entity) => {
            this.teleport(entity.location, { rotation: this.getRotation() });
            entity.remove();
            this.removeEffect("invisibility");
        });
        utils.waitForTicks(5).then(() => {
            if (this.isValid() && this.Properties.TimeAlive === 0) {
                this.Properties.TimeAlive = Date.now();
            }
        });
        if ((_a = utils.readDynamicProperty(mc.world, "ldz_240913:retaliate_against_players")) !== null && _a !== void 0 ? _a : true) {
            this.addTag("dont_retaliate_against_players");
        }
        else {
            this.removeTag("dont_retaliate_against_players");
        }
        utils.mclog("Bruce Lee constructed", LOGTAG);
        if (this.Properties.OwnerId) {
            utils.getAllPlayersSafe().forEach((player) => {
                var _a;
                if (player.id === this.Properties.OwnerId) {
                    utils.mclog(`Bruce Lee owner set to ${player.name}`);
                    this.Owner = player;
                    let followRange = (_a = readDynamicProperty(player, "party_follow_range")) !== null && _a !== void 0 ? _a : 15;
                    this.triggerEvent(`set_leash_distance_${followRange}`);
                }
            });
            this.addActorComponent(BackupComponent, {
                backupEvents: ["save"],
                deleteBackupOnKilled: false,
                saveCategory: this.Properties.OwnerId,
                categoryLimit: 0
            });
        }
    }
    tame(player) {
        var _a, _b;
        utils.mclog(`Bruce Lee owner set to ${player.name}`);
        (_a = this.getComponent("tameable")) === null || _a === void 0 ? void 0 : _a.tame(player.UnsafeEntity);
        this.Owner = player;
        this.addActorComponent(BackupComponent, {
            backupEvents: ["save"],
            deleteBackupOnKilled: false,
            saveCategory: player.id,
            categoryLimit: 0
        });
        this.Properties.Retired = false;
        let followRange = (_b = readDynamicProperty(player, "party_follow_range")) !== null && _b !== void 0 ? _b : 15;
        this.triggerEvent(`set_leash_distance_${followRange}`);
        this.raiseEvent("save");
    }
    chargeDragonFury() {
        if (this.Properties.DragonFuryCharge < 100 &&
            this.Properties.CanAttack &&
            this.Properties.CanChargeDragonFury) {
            this.Properties.DragonFuryCharge += 1;
        }
    }
    onMeditate() {
        utils.mclog("Bruce Lee is meditating", LOGTAG);
        this.Properties.TimeMeditating += 2;
        this.addEffect("slowness", 30, { amplifier: 100, showParticles: false });
        if (this.Properties.Health < this.Properties.MaxHealth) {
            this.Properties.Health += 1;
        }
        else {
            this.Properties.Meditating = false;
        }
        this.IdleLines.startTimer();
    }
    onHurt(event) {
        var _a, _b, _c, _d;
        if (event.hurtEntity.id !== this.id)
            return;
        utils.mclog(`Took ${event.damage} damage`, LOGTAG);
        if (event.damage === 10000) {
            (_a = this.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound("die");
            this.remove();
            return;
        }
        (_b = this.Health) === null || _b === void 0 ? void 0 : _b.resetToMaxValue();
        if (!this.Properties.CanAttack && !this.Properties.Meditating)
            return;
        let damage = event.damage;
        this.Properties.DamageTaken += damage;
        if (this.Properties.Health - damage > 0) {
            (_c = this.SoundPlayer) === null || _c === void 0 ? void 0 : _c.playSound("hurt");
            this.Properties.Health -= damage;
            if (event.damageSource.cause === "magic") {
                let clouds = utils.getEntitiesSafe(this.dimension, { type: "minecraft:area_effect_cloud", location: this.location, maxDistance: 6 });
                if (clouds.length > 0) {
                    let cloud = clouds[0];
                    let direction = this.location.subtract(cloud.location);
                    this.applyImpulse(direction.normalize().multiply(1));
                    utils.mclog("Impulse applied", LOGTAG);
                }
            }
            else if (((_d = event.damageSource.damagingEntity) === null || _d === void 0 ? void 0 : _d.typeId) === "minecraft:player") {
                let priority = this.Properties.FightingPriority;
                utils.writeDynamicProperty(this, "should_reset_priority", priority);
                utils.waitForTicks(100).then(() => {
                    if (!this.isValid())
                        return;
                    utils.mclog("Bruce Lee is distracted", LOGTAG);
                    this.Properties.FightingPriority = "passive";
                    this.triggerEvent("reset_attack");
                    if (!this.hasTag("ldz_240913:dont_retaliate_against_players")) {
                        this.addTag("ldz_240913:dont_retaliate_against_players");
                        utils.waitForTicks(5).then(() => {
                            if (!this.isValid())
                                return;
                            this.removeTag("ldz_240913:dont_retaliate_against_players");
                        });
                    }
                    utils.waitForTicks(25).then(() => {
                        if (!this.isValid())
                            return;
                        this.Properties.FightingPriority = priority;
                        this.dimension.spawnEntity("ldz_240913:bruce_distraction", this.location);
                    });
                });
            }
        }
        else {
            this.onDeath();
        }
        this.Properties.Meditating = false;
    }
    onBlock() {
        utils.mclog("Bruce Lee is blocking", LOGTAG);
        this.Properties.BlockedDamage = true;
        this.Properties.AttacksBlocked++;
        this.dimension.playSound("ldz_240913:bruce_lee.block", this.location);
        this.addEffect("slowness", BLOCK_DURATION, {
            amplifier: 255,
            showParticles: false
        });
        utils.waitForEvent(BLOCK_DURATION, () => {
            this.Properties.BlockedDamage = false;
        });
    }
    async onDeath() {
        var _a, _b, _c;
        this.IdleLines.stopTimer();
        (_a = this.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound("die");
        this.sendDeathMessage();
        this.Properties.Health = 0;
        this.Properties.Experience = 0;
        this.Properties.CanAttack = false;
        this.Properties.TimesKnockedOut += 1;
        if (this.Owner && !isAchievementCompleted(this.Owner, "defeat_50_opponents_consecutively")) {
            utils.mclog("Reset defeat_50_opponents_consecutively achievement");
            setAchievementValue(new utils.SafetyCheckedPlayer(this.Owner), "defeat_50_opponents_consecutively", 0);
        }
        utils.mclog("Bruce Lee died", LOGTAG);
        let dead = utils.spawnEntitySafe(this.dimension, "ldz_240913:dead_bruce_lee", this.location, { rotation: this.getRotation() });
        dead.setProperty("ldz_240913:outfit", this.Properties.Outfit);
        dead.setProperty("ldz_240913:martial_art", this.Properties.MartialArt);
        dead.setProperty("ldz_240913:mood", DEAD_MOOD);
        dead.setProperty("ldz_240913:hairstyle", this.Properties.Hairstyle);
        dead.addTag(`ldz_240913:${this.id}`);
        utils.waitForEvent(1, () => dead.applyDamage(0.1, { cause: mc.EntityDamageCause.selfDestruct }));
        this.addEffect("invisibility", 143, { showParticles: false });
        this.addEffect("slowness", 155, { amplifier: 100, showParticles: false });
        this.triggerEvent("die");
        this.triggerEvent("remove_leash");
        let startTime = mc.system.currentTick;
        utils.waitForLoopEvent(0, () => {
            if (!this.isValid())
                return true;
            this.teleport({ x: this.location.x, y: Math.min(this.dimension.heightRange.max - 10, this.location.y + 100), z: this.location.z }, { rotation: this.getRotation() });
            return mc.system.currentTick - startTime >= 100;
        });
        await utils.waitForTicks(100);
        if (!this.isValid())
            return;
        let randomRevive = Math.random() < 0.5 ? "revive.1" : "revive.2";
        (_b = this.SoundPlayer) === null || _b === void 0 ? void 0 : _b.playSound(randomRevive);
        dead.playAnimation(`animation.ldz_240913.bruce_lee.start_${randomRevive}`);
        await utils.waitForTicks(20);
        if (!this.isValid())
            return;
        dead.setProperty("ldz_240913:mood", this.Properties.PreviousMood);
        dead.setProperty("ldz_240913:outfit", this.Properties.Outfit % 11);
        utils.waitForLoopEvent(1, () => {
            if (!this.isValid())
                return true;
            this.teleport(dead.location, { rotation: this.getRotation() });
            return mc.system.currentTick - startTime >= 140;
        });
        await utils.waitForTicks(20);
        if (!this.isValid())
            return;
        dead.remove();
        this.Properties.Health = this.Properties.MaxHealth / 2;
        if (!this.isValid())
            return;
        utils.mclog("Bruce Lee revived", LOGTAG);
        this.Properties.CanAttack = true;
        this.triggerEvent("revive");
        if (this.Owner) {
            let followRange = (_c = readDynamicProperty(this.Owner, "party_follow_range")) !== null && _c !== void 0 ? _c : 15;
            this.triggerEvent(`set_leash_distance_${followRange}`);
        }
        this.IdleLines.startTimer();
    }
    sendDeathMessage() {
        var _a;
        if (this.Owner) {
            const prop = (_a = utils.readDynamicProperty(this.Owner, "ldz_240913:send_death_messages")) !== null && _a !== void 0 ? _a : true;
            if (prop) {
                this.Owner.sendMessage({
                    translate: "ldz_240913:chat_message.bruce_lee_knocked_out"
                });
            }
        }
    }
    takeABreak() {
        var _a;
        this.Properties.CanAttack = false;
        this.Properties.Retired = true;
        this.IdleLines.stopTimer();
        this.triggerEvent("die");
        this.addEffect("slowness", 100, { amplifier: 100, showParticles: false });
        this.setProperty("ldz_240913:retiring", true);
        this.playAnimation("animation.ldz_240913.bruce_lee.retire");
        (_a = this.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound("take_a_break");
        this.raiseEvent("save");
        utils.waitForEvent(64, () => {
            const spawnEgg = new mc.ItemStack("ldz_240913:bruce_lee_spawn_egg", 1);
            spawnEgg.setDynamicProperty("ldz_240913:bruce_lee_id", this.id);
            spawnEgg.setLore(this.getRetiredLore());
            this.dimension.spawnItem(spawnEgg, this.location);
            this.remove();
        });
        if (this.Properties.Level === this.Properties.HighestLevelBruceLee) {
            let nextHighest = 0;
            utils.Actor.getActors(BruceLee).forEach((actor) => {
                if (actor.isValid() && actor.Properties.Level > nextHighest) {
                    nextHighest = actor.Properties.Level;
                }
            });
            this.Properties.HighestLevelBruceLee = nextHighest;
        }
    }
    getRetiredLore() {
        let outfit = "";
        let martialArt = "";
        utils.mclog(`Outfit: ${this.Properties.Outfit % 11}`, LOGTAG);
        switch (this.Properties.Outfit % 11) {
            case 0:
                outfit = "Boss Sash";
                break;
            case 1:
                outfit = "Wing Chun Uniform";
                break;
            case 2:
                outfit = "Brown Kung Fu Uniform";
                break;
            case 3:
                outfit = "Green Tangzhuang";
                break;
            case 4:
                outfit = "Formal Black Suit";
                break;
            case 5:
                outfit = "Red Tracksuit";
                break;
            case 6:
                outfit = "Way of the Tank Top";
                break;
            case 7:
                outfit = "White & Blue Kung Fu Uniform";
                break;
            case 8:
                outfit = "Formal White Suit";
                break;
            case 9:
                outfit = "Yellow Jumpsuit";
                break;
            case 10:
                outfit = "Dragon Scars";
                break;
        }
        switch (this.Properties.MartialArt) {
            case "tabak_toyok":
                martialArt = "Tabak Toyok";
                break;
            case "bo_staff":
                martialArt = "Bo Staff";
                break;
            case "escrima_sticks":
                martialArt = "Escrima Sticks";
                break;
            default:
                martialArt = "Unarmed";
                break;
        }
        return [`§r§fLevel: §5${this.Properties.Level}§r`, `§r§fOutfit: §5${outfit}§r`, `§r§fMartial Art: §5${martialArt}§r`];
    }
    gainExperience(amount) {
        var _a;
        if (this.Properties.Level >= 10 && this.Properties.MartialArt === "none")
            return;
        if (this.Owner) {
            this.playAnimation("animation.ldz_240913.bruce_lee_hud.show", {
                players: [this.Owner.name]
            });
        }
        this.Properties.Experience += amount;
        if (this.Properties.Experience + amount >=
            this.Properties.ExperienceToNextLevel) {
            this.levelUp();
        }
        else {
            (_a = this.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound("gain_experience");
        }
    }
    levelUp() {
        var _a, _b, _c;
        if (this.Properties.Level >= 40)
            return;
        this.setLevelUpProperties();
        if (this.Properties.Level === 9) {
            (_a = this.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound("level_10");
            if (this.Owner) {
                (_b = this.SoundPlayer) === null || _b === void 0 ? void 0 : _b.playSound("level_10", this.Owner.dimension, this.Owner.location);
                this.Owner.sendMessage({ translate: "ldz_240913:chat_message.bruce_lee_level_10" });
            }
        }
        else {
            (_c = this.SoundPlayer) === null || _c === void 0 ? void 0 : _c.playSound("level_up");
        }
    }
    async setLevelUpProperties() {
        await utils.waitForTicks(1);
        while (this.Properties.Experience >= this.Properties.ExperienceToNextLevel) {
            this.Properties.Experience -= this.Properties.ExperienceToNextLevel;
            this.Properties.Level += 1;
            this.Properties.SkillPoints += 1;
            this.Properties.ExperienceToNextLevel = utils.clamp(this.Properties.ExperienceToNextLevel * 1.25 + 1, 0, 10000);
            await utils.waitForTicks(2);
        }
        this.Properties.Health = this.Properties.MaxHealth;
        utils.waitForEvent(1, () => {
            this.Properties.Health = this.Properties.MaxHealth;
        });
    }
    destructor() {
        mc.world.afterEvents.entityHurt.unsubscribe(this.onHurtBinding);
        mc.system.afterEvents.scriptEventReceive.unsubscribe(this.scriptEventBinding);
        super.destructor();
    }
}
(() => {
    mc.world.afterEvents.entityLoad.subscribe((event) => {
        if (event.entity.typeId === "ldz_240913:bruce_lee") {
            utils.waitForEvent(5, () => {
                const bruce = BruceLee.getActor(event.entity.id, BruceLee);
                if (!bruce)
                    return;
                if (bruce.Properties.Health <= 0) {
                    utils.mclog(`Loaded Bruce with health ${bruce === null || bruce === void 0 ? void 0 : bruce.Properties.Health}`, LOGTAG);
                    bruce.Properties.Health = bruce.Properties.MaxHealth / 2;
                    bruce.triggerEvent("revive");
                }
                const priority = utils.readDynamicProperty(bruce, "should_reset_priority");
                if (priority) {
                    bruce.Properties.FightingPriority = priority;
                }
            });
        }
    });
})();
// test level up
mc.system.afterEvents.scriptEventReceive.subscribe((event) => {
    if (event.id === "ldz_240913:bruce_lee_level_up" && event.sourceEntity) {
        const blee = utils.Actor.getActor(event.sourceEntity.id, BruceLee);
        blee === null || blee === void 0 ? void 0 : blee.levelUp();
    }
});
BruceLee.makeActors({ type: "ldz_240913:bruce_lee" }, BruceLee);
mc.world.afterEvents.entitySpawn.subscribe(async (event) => {
    if (event.entity.typeId === "ldz_240913:bruce_lee") {
        const afterUseBruceLeeSpawnEgg = async (itemEvent) => {
            if (itemEvent.itemStack.typeId === "ldz_240913:bruce_lee_spawn_egg") {
                let linked_id = itemEvent.itemStack.getDynamicProperty("ldz_240913:bruce_lee_id");
                utils.mclog(`Linked id: ${linked_id}`, LOGTAG);
                await utils.waitForTicks(2);
                if (linked_id) {
                    BackupSystem.loadFromBackup(linked_id, event.entity.dimension, new utils.Vector3(event.entity.location), false).then(async (actor) => {
                        if (actor) {
                            actor.Properties.OwnerId = itemEvent.source.id;
                        }
                        else {
                            let actor = utils.Actor.getActor(event.entity.id, BruceLee);
                            if (actor) {
                                actor.Properties.OwnerId = itemEvent.source.id;
                                actor.setDynamicProperty("ldz_240913:retiring", false);
                            }
                        }
                    });
                    BackupSystem.removeBackup(event.entity.id);
                    event.entity.remove();
                }
                else {
                    await utils.waitForTicks(2);
                    let actor = utils.Actor.getActor(event.entity.id, BruceLee);
                    if (actor) {
                        actor.Properties.OwnerId = itemEvent.source.id;
                    }
                }
            }
        };
        mc.world.afterEvents.itemUseOn.subscribe(afterUseBruceLeeSpawnEgg);
        await utils.waitForTicks(1);
        mc.world.afterEvents.itemUseOn.unsubscribe(afterUseBruceLeeSpawnEgg);
    }
});
