import { system } from "@minecraft/server";
import { ActorComponent, abortEvents, getEntitiesSafe, mclog, waitForEvent, waitForLoopEvent } from "../utils/index";
export class BruceLeeIdleLinesComponent extends ActorComponent {
    constructor() {
        super(...arguments);
        this.MAX_IDLE_TIME = 380;
        this.MIN_IDLE_TIME = 55;
        this.idleLines = [
            { line: "bits.be_formless", length: 1.2 },
            { line: "bits.be_like_water", length: 1.2 },
            { line: "bits.be_shapeless", length: 1 },
            { line: "bits.be_water_my_friend", length: 1.5 },
            { line: "bits.i_do_not_believe", length: 2.5 },
            { line: "bits.keep_on_flowing", length: 1.1 },
            { line: "bits.laugh", length: 1.9 },
            { line: "bits.make_this_into_a_weapon", length: 1.4 },
            { line: "bits.running_water_never", length: 2.4 },
            { line: "bits.snap_it", length: 0.8 },
            { line: "bits.ultimately_martial_art", length: 5 },
            { line: "bits.we_have_only_2_hands", length: 1.6 },
            { line: "bits.you_have_to_train", length: 1.2 },
            { line: "bits.baby_you_better_train", length: 2.5 },
            { line: "bits.empty_your_mind", length: 2 },
            { line: "bits.here_i_am", length: 2 },
            { line: "bits.here_is_control", length: 1.2 },
            { line: "bits.martial_art_has", length: 3 },
        ];
        this.idleLine = 0;
    }
    startTimer() {
        abortEvents(this.actor.id);
        let randomTime = Math.floor(Math.random() * (this.MAX_IDLE_TIME - this.MIN_IDLE_TIME) + this.MIN_IDLE_TIME) * 20;
        waitForLoopEvent(randomTime, async () => {
            var _a;
            if ((_a = this.actor.Owner) === null || _a === void 0 ? void 0 : _a.hasTag("disable_idle_lines")) {
                abortEvents(this.actor.id);
                return true;
            }
            waitForEvent(randomTime, () => {
                var _a, _b;
                if ((_a = this.actor.Owner) === null || _a === void 0 ? void 0 : _a.hasTag("disable_idle_lines")) {
                    abortEvents(this.actor.id);
                    return true;
                }
                this.actor.addTag("talking");
                const nearbyTalkingActors = getEntitiesSafe(this.actor.dimension, { location: this.actor.location, tags: ["ldz_240913:talking"], maxDistance: 20 });
                if (nearbyTalkingActors.length > 1) {
                    this.actor.removeTag("talking");
                    return;
                }
                if (this.idleLine === 0) {
                    let previousLastLine = this.idleLines[this.idleLines.length - 1].line;
                    this.idleLines.sort(() => Math.random() - 0.5);
                    if (this.idleLines[0].line === previousLastLine) {
                        this.idleLines.reverse();
                    }
                }
                mclog(` [${system.currentTick / 20}] Playing idle line ${this.idleLine}`, "BruceLee");
                (_b = this.actor.SoundPlayer) === null || _b === void 0 ? void 0 : _b.playSound(this.idleLines[this.idleLine].line);
                this.actor.playAnimation("animation.ldz_240913.bruce_lee.start_talking", {
                    "controller": "animation.ldz_240913.bruce_lee.talking.controller"
                });
                waitForEvent(this.idleLines[this.idleLine].length * 20, () => {
                    this.actor.playAnimation("animation.ldz_240913.bruce_lee.stop_talking", {
                        "controller": "animation.ldz_240913.bruce_lee.talking.controller"
                    });
                    this.actor.removeTag("talking");
                    this.idleLine = (this.idleLine + 1) % this.idleLines.length;
                }, this.actor.id);
                randomTime = (Math.random() * (this.MAX_IDLE_TIME - this.MIN_IDLE_TIME) + this.MIN_IDLE_TIME + this.idleLines[this.idleLine].length) * 20;
            }, this.actor.id);
            return false;
        }, this.actor.id);
    }
    stopTimer() {
        abortEvents(this.actor.id);
        this.actor.playAnimation("animation.ldz_240913.bruce_lee.stop_talking", {
            "controller": "animation.ldz_240913.bruce_lee.talking.controller"
        });
    }
    onComponentAdded() {
        this.startTimer();
    }
    onComponentRemoved() {
        this.stopTimer();
    }
}
