import * as utils from "./../utils/index";
import { system, world } from "@minecraft/server";
import { mclog, MenuBuilder, readDynamicProperty, waitForEvent, waitForLoopEvent, writeDynamicProperty, overworld, nether, end } from "../utils/index";
import { ActorAttack } from "./attack";
import { BO_STAFF_DEFAULT_ATTACKS } from "./bo_staff_attacks";
import { BruceLee } from "./bruce_lee";
import { ESCRIMA_STICKS_DEFAULT_ATTACKS } from "./escrima_sticks_attacks";
import { TABAK_TOYOK_DEFAULT_ATTACKS } from "./tabak_toyok_attacks";
import { checkAchievement, getAchievementsMenuBodyText, getCompletedAchievements, incrementAchievement } from "../achievements";
import { OpponentSoundPlayer } from "../opponents/sound_player";
import { BruceLeeSoundPlayer } from "./sound_player";
const COST_MAP = {
    health: [
        1,
        1,
        2,
        2,
        2,
        3,
        3,
        3,
        3,
        3,
        -1
    ],
    damage: [
        1,
        1,
        2,
        2,
        2,
        2,
        3,
        3,
        3,
        3,
        -1
    ],
    speed: [
        1,
        1,
        2,
        2,
        2,
        2,
        3,
        3,
        3,
        3,
        -1
    ],
    block_chance: [
        1,
        1,
        2,
        2,
        2,
        3,
        3,
        3,
        3,
        3,
        -1
    ],
    dragon_fury: [
        2,
        3,
        -1
    ]
};
const COST_LANG_MAP = {
    health: [
        "ldz_240913:menu.bruce_lee.martial_arts.one_skill_point",
        "ldz_240913:menu.bruce_lee.martial_arts.one_skill_point",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.fully_upgraded"
    ],
    damage: [
        "ldz_240913:menu.bruce_lee.martial_arts.one_skill_point",
        "ldz_240913:menu.bruce_lee.martial_arts.one_skill_point",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.fully_upgraded"
    ],
    speed: [
        "ldz_240913:menu.bruce_lee.martial_arts.one_skill_point",
        "ldz_240913:menu.bruce_lee.martial_arts.one_skill_point",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.fully_upgraded"
    ],
    block_chance: [
        "ldz_240913:menu.bruce_lee.martial_arts.one_skill_point",
        "ldz_240913:menu.bruce_lee.martial_arts.one_skill_point",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.fully_upgraded"
    ],
    dragon_fury: [
        "ldz_240913:menu.bruce_lee.martial_arts.two_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.three_skill_points",
        "ldz_240913:menu.bruce_lee.martial_arts.fully_upgraded"
    ]
};
const FULLY_UPGRADED = "ldz_240913:menu.bruce_lee.martial_arts.fully_upgraded";
export async function showMenuToPlayer(actor, player, path) {
    //const player_version = readDynamicProperty<string>(player, "version") ?? "1.0.0"
    const healthLevel = actor.Properties.MaxHealth % 20;
    const damageLevel = actor.Properties.AttackDamage - 1;
    const speedLevel = actor.Properties.Speed - 1;
    const blockChanceLevel = actor.Properties.BlockChance / 5;
    const dragonFuryLevel = actor.Properties.DragonFuryLevel - 1;
    const healthCost = COST_MAP.health[healthLevel];
    const damageCost = COST_MAP.damage[damageLevel];
    const speedCost = COST_MAP.speed[speedLevel];
    const blockChanceCost = COST_MAP.block_chance[blockChanceLevel];
    const dragonFuryCost = COST_MAP.dragon_fury[dragonFuryLevel];
    const healthLang = COST_LANG_MAP.health[healthLevel];
    const damageLang = COST_LANG_MAP.damage[damageLevel];
    const speedLang = COST_LANG_MAP.speed[speedLevel];
    const blockChanceLang = COST_LANG_MAP.block_chance[blockChanceLevel];
    const dragonFuryLang = COST_LANG_MAP.dragon_fury[dragonFuryLevel];
    let selectedMartialArt = "none";
    let martial_arts_texture = "textures/ldz/240913/ui/unarmed";
    let damage_texture = "textures/ldz/240913/ui/unarmed_plus";
    switch (actor.Properties.MartialArt) {
        case "none":
            if (actor.Properties.Level >= 10) {
                martial_arts_texture = "textures/ldz/240913/ui/new";
            }
            break;
        case "unarmed":
            martial_arts_texture = "textures/ldz/240913/ui/unarmed";
            break;
        case "bo_staff":
            martial_arts_texture = "textures/ldz/240913/ui/bo_staff";
            damage_texture = "textures/ldz/240913/ui/bo_staff_plus";
            break;
        case "escrima_sticks":
            martial_arts_texture = "textures/ldz/240913/ui/escrima_sticks";
            damage_texture = "textures/ldz/240913/ui/escrima_sticks_plus";
            break;
        case "tabak_toyok":
            martial_arts_texture = "textures/ldz/240913/ui/tabak_toyok";
            damage_texture = "textures/ldz/240913/ui/tabak_toyok_plus";
            break;
    }
    let styleOptions = [
        {
            text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.white_shirt_sash_pants" },
            onSelected: () => {
                if (actor.Properties.Outfit !== 0 && actor.Properties.Outfit !== 10 && actor.Properties.Outfit !== 20) {
                    player.playSound("ldz_240913:bruce_lee.menu.change_outfit");
                }
                if (actor.Properties.Outfit > 20) {
                    actor.Properties.Outfit = 22;
                }
                else if (actor.Properties.Outfit > 10) {
                    actor.Properties.Outfit = 11;
                }
                else {
                    actor.Properties.Outfit = 0;
                }
            }
        }
    ];
    if (checkAchievement(player, "defeat_50_opponents_consecutively")) {
        styleOptions.push({
            text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.blue_wing_chun_uniform" },
            onSelected: () => {
                if (actor.Properties.Outfit !== 1 && actor.Properties.Outfit !== 11 && actor.Properties.Outfit !== 21) {
                    player.playSound("ldz_240913:bruce_lee.menu.change_outfit");
                }
                if (actor.Properties.Outfit > 20) {
                    actor.Properties.Outfit = 23;
                }
                else if (actor.Properties.Outfit > 10) {
                    actor.Properties.Outfit = 12;
                }
                else {
                    actor.Properties.Outfit = 1;
                }
            }
        });
    }
    if (checkAchievement(player, "meditate_near_warden")) {
        styleOptions.push({
            text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.brown_tai_chi_uniform" },
            onSelected: () => {
                if (actor.Properties.Outfit !== 2 && actor.Properties.Outfit !== 12 && actor.Properties.Outfit !== 22) {
                    player.playSound("ldz_240913:bruce_lee.menu.change_outfit");
                }
                if (actor.Properties.Outfit > 20) {
                    actor.Properties.Outfit = 24;
                }
                else if (actor.Properties.Outfit > 10) {
                    actor.Properties.Outfit = 13;
                }
                else {
                    actor.Properties.Outfit = 2;
                }
            }
        });
    }
    if (checkAchievement(player, "kill_creeper")) {
        styleOptions.push({
            text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.green" },
            onSelected: () => {
                if (actor.Properties.Outfit !== 3 && actor.Properties.Outfit !== 13 && actor.Properties.Outfit !== 23) {
                    player.playSound("ldz_240913:bruce_lee.menu.change_outfit");
                }
                if (actor.Properties.Outfit > 20) {
                    actor.Properties.Outfit = 25;
                }
                else if (actor.Properties.Outfit > 10) {
                    actor.Properties.Outfit = 14;
                }
                else {
                    actor.Properties.Outfit = 3;
                }
            }
        });
    }
    if (checkAchievement(player, "birthday")) {
        styleOptions.push({
            text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.formal_black_suit" },
            onSelected: () => {
                if (actor.Properties.Outfit !== 4 && actor.Properties.Outfit !== 14 && actor.Properties.Outfit !== 24) {
                    player.playSound("ldz_240913:bruce_lee.menu.change_outfit");
                }
                if (actor.Properties.Outfit > 20) {
                    actor.Properties.Outfit = 26;
                }
                else if (actor.Properties.Outfit > 10) {
                    actor.Properties.Outfit = 15;
                }
                else {
                    actor.Properties.Outfit = 4;
                }
            }
        });
    }
    if (checkAchievement(player, "block_attacks")) {
        styleOptions.push({
            text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.red_tracksuit" },
            onSelected: () => {
                if (actor.Properties.Outfit !== 5 && actor.Properties.Outfit !== 15 && actor.Properties.Outfit !== 25) {
                    player.playSound("ldz_240913:bruce_lee.menu.change_outfit");
                }
                if (actor.Properties.Outfit > 20) {
                    actor.Properties.Outfit = 27;
                }
                else if (actor.Properties.Outfit > 10) {
                    actor.Properties.Outfit = 16;
                }
                else {
                    actor.Properties.Outfit = 5;
                }
            }
        });
    }
    if (checkAchievement(player, "defeat_opponents")) {
        styleOptions.push({
            text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.tank_top" },
            onSelected: () => {
                if (actor.Properties.Outfit !== 6 && actor.Properties.Outfit !== 16 && actor.Properties.Outfit !== 26) {
                    player.playSound("ldz_240913:bruce_lee.menu.change_outfit");
                }
                if (actor.Properties.Outfit > 20) {
                    actor.Properties.Outfit = 28;
                }
                else if (actor.Properties.Outfit > 10) {
                    actor.Properties.Outfit = 17;
                }
                else {
                    actor.Properties.Outfit = 6;
                }
            }
        });
    }
    if (checkAchievement(player, "activate_dragon_fury")) {
        styleOptions.push({
            text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.white_kung_fu_uniform" },
            onSelected: () => {
                if (actor.Properties.Outfit !== 7 && actor.Properties.Outfit !== 17 && actor.Properties.Outfit !== 27) {
                    player.playSound("ldz_240913:bruce_lee.menu.change_outfit");
                }
                if (actor.Properties.Outfit > 20) {
                    actor.Properties.Outfit = 29;
                }
                else if (actor.Properties.Outfit > 10) {
                    actor.Properties.Outfit = 18;
                }
                else {
                    actor.Properties.Outfit = 7;
                }
            }
        });
    }
    if (checkAchievement(player, "complete_martial_arts")) {
        styleOptions.push({
            text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.formal_white_suit" },
            onSelected: () => {
                if (actor.Properties.Outfit !== 8 && actor.Properties.Outfit !== 18 && actor.Properties.Outfit !== 28) {
                    player.playSound("ldz_240913:bruce_lee.menu.change_outfit");
                }
                if (actor.Properties.Outfit > 20) {
                    actor.Properties.Outfit = 30;
                }
                else if (actor.Properties.Outfit > 10) {
                    actor.Properties.Outfit = 19;
                }
                else {
                    actor.Properties.Outfit = 8;
                }
            }
        });
    }
    if (checkAchievement(player, "max_out_skills")) {
        styleOptions.push({
            text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.yellow_jumpsuit" },
            onSelected: () => {
                if (actor.Properties.Outfit !== 9 && actor.Properties.Outfit !== 19 && actor.Properties.Outfit !== 29) {
                    player.playSound("ldz_240913:bruce_lee.menu.change_outfit");
                }
                if (actor.Properties.Outfit > 20) {
                    actor.Properties.Outfit = 31;
                }
                else if (actor.Properties.Outfit > 10) {
                    actor.Properties.Outfit = 20;
                }
                else {
                    actor.Properties.Outfit = 9;
                }
            }
        });
    }
    if (checkAchievement(player, "defeat_50_vanilla_mobs")) {
        styleOptions.push({
            text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.shirtless" },
            onSelected: () => {
                if (actor.Properties.Outfit !== 10 && actor.Properties.Outfit !== 20) {
                    player.playSound("ldz_240913:bruce_lee.menu.change_outfit");
                }
                if (actor.Properties.Outfit > 20) {
                    actor.Properties.Outfit = 32;
                }
                else if (actor.Properties.Outfit > 10) {
                    actor.Properties.Outfit = 21;
                }
                else {
                    actor.Properties.Outfit = 10;
                }
            }
        });
    }
    const menu = new MenuBuilder({
        type: "page_list",
        title: { translate: "ldz_240913:menu.bruce_lee.title", with: [`${actor.Properties.Level}`] },
        contents: {
            martialArts: (actor.Properties.Level >= 10 && actor.Properties.MartialArt !== "none") ? {
                type: "buttons",
                buttonLabel: { translate: "ldz_240913:menu.bruce_lee.martial_arts.button" },
                icon: martial_arts_texture,
                title: { translate: "ldz_240913:menu.bruce_lee.martial_arts.title" },
                body: { translate: "ldz_240913:menu.bruce_lee.martial_arts.body", with: ["\n", `${actor.Properties.SkillPoints}`] },
                ignoreBookmark: true,
                contents: [
                    {
                        text: { translate: "ldz_240913:menu.bruce_lee.martial_arts.upgrade_health", with: { rawtext: [{ text: "\n" }, { translate: healthLang, with: [`${healthLevel + 1}`] }] } },
                        texture: "textures/ldz/240913/ui/upgrade_health",
                        onClick: () => {
                            if (healthCost === -1) {
                                waitForEvent(1, () => {
                                    showMenuToPlayer(actor, player, "root/martialArts");
                                });
                                return;
                            }
                            if (actor.Properties.SkillPoints >= healthCost) {
                                actor.Properties.MaxHealth += 1;
                                actor.Properties.SkillPoints -= healthCost;
                                player.playSound("ldz_240913:bruce_lee.menu.upgrade_health");
                                waitForEvent(1, () => {
                                    showMenuToPlayer(actor, player, "root/martialArts");
                                });
                                if (COST_MAP.health[healthLevel + 1] === -1) {
                                    incrementAchievement(player, "max_out_skills");
                                }
                            }
                            else {
                                showNotEnoughSkillPointsMenu(actor, player, `${healthCost}`, `${actor.Properties.MaxHealth}`, "ldz_240913:menu.bruce_lee.martial_arts.health", "60");
                                player.playSound("ldz_240913:bruce_lee.menu.fail");
                            }
                        }
                    },
                    {
                        text: { translate: "ldz_240913:menu.bruce_lee.martial_arts.upgrade_damage", with: { rawtext: [{ text: "\n" }, { translate: damageLang, with: [`${damageLevel + 1}`] }] } },
                        texture: damage_texture,
                        onClick: () => {
                            if (damageCost === -1) {
                                waitForEvent(1, () => {
                                    showMenuToPlayer(actor, player, "root/martialArts");
                                });
                                return;
                            }
                            if (actor.Properties.SkillPoints >= damageCost) {
                                actor.Properties.AttackDamage += 1;
                                actor.Properties.SkillPoints -= damageCost;
                                player.playSound("ldz_240913:bruce_lee.menu.upgrade_damage");
                                waitForEvent(1, () => {
                                    showMenuToPlayer(actor, player, "root/martialArts");
                                });
                                if (COST_MAP.damage[damageLevel + 1] === -1) {
                                    incrementAchievement(player, "max_out_skills");
                                }
                            }
                            else {
                                showNotEnoughSkillPointsMenu(actor, player, `${damageCost}`, `${actor.Properties.AttackDamage}`, "ldz_240913:menu.bruce_lee.martial_arts.damage", "11");
                                player.playSound("ldz_240913:bruce_lee.menu.fail");
                            }
                        }
                    },
                    {
                        text: { translate: "ldz_240913:menu.bruce_lee.martial_arts.upgrade_speed", with: { rawtext: [{ text: "\n" }, { translate: speedLang, with: [`${speedLevel + 1}`] }] } },
                        texture: "textures/ldz/240913/ui/upgrade_speed",
                        onClick: () => {
                            if (speedCost === -1) {
                                waitForEvent(1, () => {
                                    showMenuToPlayer(actor, player, "root/martialArts");
                                });
                                return;
                            }
                            if (actor.Properties.SkillPoints >= speedCost) {
                                actor.Properties.Speed += 1;
                                actor.Properties.SkillPoints -= speedCost;
                                player.playSound("ldz_240913:bruce_lee.menu.upgrade_speed");
                                waitForEvent(1, () => {
                                    showMenuToPlayer(actor, player, "root/martialArts");
                                });
                                if (COST_MAP.speed[speedLevel + 1] === -1) {
                                    incrementAchievement(player, "max_out_skills");
                                }
                            }
                            else {
                                showNotEnoughSkillPointsMenu(actor, player, `${speedCost}`, `${actor.Properties.Speed}`, "ldz_240913:menu.bruce_lee.martial_arts.speed", "2");
                                player.playSound("ldz_240913:bruce_lee.menu.fail");
                            }
                        }
                    },
                    {
                        text: { translate: "ldz_240913:menu.bruce_lee.martial_arts.upgrade_block_chance", with: { rawtext: [{ text: "\n" }, { translate: blockChanceLang, with: [`${blockChanceLevel + 1}`] }] } },
                        texture: "textures/ldz/240913/ui/upgrade_block_chance",
                        onClick: () => {
                            if (blockChanceCost === -1) {
                                waitForEvent(1, () => {
                                    showMenuToPlayer(actor, player, "root/martialArts");
                                });
                                return;
                            }
                            if (actor.Properties.SkillPoints >= blockChanceCost) {
                                actor.Properties.BlockChance += 5;
                                actor.Properties.SkillPoints -= blockChanceCost;
                                player.playSound("ldz_240913:bruce_lee.menu.upgrade_block_chance");
                                waitForEvent(1, () => {
                                    showMenuToPlayer(actor, player, "root/martialArts");
                                });
                                if (COST_MAP.block_chance[blockChanceLevel + 1] === -1) {
                                    incrementAchievement(player, "max_out_skills");
                                }
                            }
                            else {
                                showNotEnoughSkillPointsMenu(actor, player, `${blockChanceCost}`, `${actor.Properties.BlockChance}%`, "ldz_240913:menu.bruce_lee.martial_arts.block_chance", "50%");
                                player.playSound("ldz_240913:bruce_lee.menu.fail");
                            }
                        }
                    },
                    {
                        text: { translate: "ldz_240913:menu.bruce_lee.martial_arts.upgrade_dragon_fury", with: { rawtext: [{ text: "\n" }, { translate: dragonFuryLang, with: [`${dragonFuryLevel + 2}`] }] } },
                        texture: "textures/ldz/240913/ui/upgrade_dragon_fury",
                        onClick: () => {
                            if (dragonFuryCost === -1) {
                                waitForEvent(1, () => {
                                    showMenuToPlayer(actor, player, "root/martialArts");
                                });
                                return;
                            }
                            if (actor.Properties.SkillPoints >= dragonFuryCost) {
                                actor.Properties.DragonFuryLevel += 1;
                                actor.Properties.SkillPoints -= dragonFuryCost;
                                player.playSound("ldz_240913:bruce_lee.menu.upgrade_dragon_fury");
                                waitForEvent(1, () => {
                                    showMenuToPlayer(actor, player, "root/martialArts");
                                });
                                if (COST_MAP.dragon_fury[dragonFuryLevel + 1] === -1) {
                                    incrementAchievement(player, "max_out_skills");
                                }
                            }
                            else {
                                showNotEnoughSkillPointsMenu(actor, player, `${dragonFuryCost}`, `Level ${dragonFuryLevel + 1}`, "ldz_240913:menu.bruce_lee.martial_arts.dragon_fury");
                                player.playSound("ldz_240913:bruce_lee.menu.fail");
                            }
                        }
                    }
                ]
            } : (actor.Properties.Level >= 10) ? {
                type: "page_list",
                title: { translate: "ldz_240913:menu.bruce_lee.martial_arts.title" },
                body: { translate: "ldz_240913:menu.bruce_lee.martial_arts.path_warning", with: ["\n"] },
                icon: "textures/ldz/240913/ui/new",
                ignoreBookmark: true,
                contents: {
                    selectPath: {
                        type: "page_list",
                        title: { translate: "ldz_240913:menu.bruce_lee.martial_arts.select_path" },
                        buttonLabel: { translate: "ldz_240913:menu.button.confirm" },
                        onPageOpen: (player) => {
                            player.playSound("ldz_240913:bruce_lee.menu.confirm");
                        },
                        icon: "textures/ui/confirm",
                        ignoreBookmark: true,
                        contents: {
                            unarmed: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.bruce_lee.martial_arts.unarmed.button" },
                                icon: "textures/ldz/240913/ui/unarmed",
                                title: { translate: "ldz_240913:menu.bruce_lee.martial_arts.unarmed" },
                                body: { translate: "ldz_240913:menu.bruce_lee.martial_arts.confirm_selection", with: ["\n"] },
                                ignoreBookmark: true,
                                contents: [
                                    {
                                        text: { translate: "ldz_240913:menu.bruce_lee.martial_arts.confirm_selection.yes" },
                                        texture: "textures/ui/confirm",
                                        onClick: () => {
                                            mclog(`Setting martial art to ${selectedMartialArt}`, "BruceLee");
                                            actor.Properties.MartialArt = "unarmed";
                                            player.playSound("ldz_240913:bruce_lee.menu.select_path_unarmed");
                                            incrementAchievement(player, "complete_martial_arts");
                                            waitForEvent(2, () => {
                                                showPathChosenMenu(actor, player);
                                            });
                                        }
                                    }
                                ]
                            },
                            bo_staff: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.bruce_lee.martial_arts.bo_staff.button" },
                                icon: "textures/ldz/240913/ui/bo_staff",
                                title: { translate: "ldz_240913:menu.bruce_lee.martial_arts.bo_staff" },
                                body: { translate: "ldz_240913:menu.bruce_lee.martial_arts.confirm_selection", with: ["\n"] },
                                ignoreBookmark: true,
                                contents: [
                                    {
                                        text: { translate: "ldz_240913:menu.bruce_lee.martial_arts.confirm_selection.yes" },
                                        texture: "textures/ui/confirm",
                                        onClick: () => {
                                            mclog(`Setting martial art to ${selectedMartialArt}`, "BruceLee");
                                            actor.Properties.MartialArt = "bo_staff";
                                            actor.removeActorComponent(ActorAttack);
                                            actor.addActorComponent(ActorAttack, BO_STAFF_DEFAULT_ATTACKS);
                                            player.playSound("ldz_240913:bruce_lee.menu.select_path_bo_staff");
                                            incrementAchievement(player, "complete_martial_arts");
                                            waitForEvent(2, () => {
                                                showPathChosenMenu(actor, player);
                                            });
                                        }
                                    }
                                ]
                            },
                            escrima_sticks: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.bruce_lee.martial_arts.escrima_sticks.button" },
                                icon: "textures/ldz/240913/ui/escrima_sticks",
                                title: { translate: "ldz_240913:menu.bruce_lee.martial_arts.escrima_sticks" },
                                body: { translate: "ldz_240913:menu.bruce_lee.martial_arts.confirm_selection", with: ["\n"] },
                                ignoreBookmark: true,
                                contents: [
                                    {
                                        text: { translate: "ldz_240913:menu.bruce_lee.martial_arts.confirm_selection.yes" },
                                        texture: "textures/ui/confirm",
                                        onClick: () => {
                                            mclog(`Setting martial art to ${selectedMartialArt}`, "BruceLee");
                                            actor.Properties.MartialArt = "escrima_sticks";
                                            actor.removeActorComponent(ActorAttack);
                                            actor.addActorComponent(ActorAttack, ESCRIMA_STICKS_DEFAULT_ATTACKS);
                                            player.playSound("ldz_240913:bruce_lee.menu.select_path_escrima_sticks");
                                            incrementAchievement(player, "complete_martial_arts");
                                            waitForEvent(2, () => {
                                                showPathChosenMenu(actor, player);
                                            });
                                        }
                                    }
                                ]
                            },
                            tabak_toyok: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.bruce_lee.martial_arts.tabak_toyok.button" },
                                icon: "textures/ldz/240913/ui/tabak_toyok",
                                title: { translate: "ldz_240913:menu.bruce_lee.martial_arts.tabak_toyok" },
                                body: { translate: "ldz_240913:menu.bruce_lee.martial_arts.confirm_selection", with: ["\n"] },
                                ignoreBookmark: true,
                                contents: [
                                    {
                                        text: { translate: "ldz_240913:menu.bruce_lee.martial_arts.confirm_selection.yes" },
                                        texture: "textures/ui/confirm",
                                        onClick: () => {
                                            mclog(`Setting martial art to ${selectedMartialArt}`, "BruceLee");
                                            actor.Properties.MartialArt = "tabak_toyok";
                                            actor.removeActorComponent(ActorAttack);
                                            actor.addActorComponent(ActorAttack, TABAK_TOYOK_DEFAULT_ATTACKS);
                                            player.playSound("ldz_240913:bruce_lee.menu.select_path_tabak_toyok");
                                            incrementAchievement(player, "complete_martial_arts");
                                            waitForEvent(2, () => {
                                                showPathChosenMenu(actor, player);
                                            });
                                        }
                                    }
                                ]
                            },
                        }
                    }
                }
            } : {
                type: "buttons",
                body: { translate: "ldz_240913:menu.bruce_lee.martial_arts.locked", with: ["\n"] },
                buttonLabel: { translate: "ldz_240913:menu.bruce_lee.martial_arts.button" },
                icon: "textures/ldz/240913/ui/unarmed",
                title: { translate: "ldz_240913:menu.bruce_lee.martial_arts.title" },
                ignoreBookmark: true,
                contents: []
            },
            fightingPriority: {
                type: "modal",
                buttonLabel: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.button" },
                icon: "textures/ldz/240913/ui/fighting_priority",
                title: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.title" },
                submitText: { translate: "ldz_240913:menu.button.confirm" },
                contents: [
                    {
                        type: "toggle",
                        text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.toggle_follow" },
                        defaultValue() {
                            return actor.Properties.StopFollowing;
                        },
                        onSubmit(player, toggleValue) {
                            var _a;
                            if (toggleValue) {
                                actor.Properties.StopFollowing = true;
                                actor.triggerEvent(`set_leash_distance_15`);
                                waitForEvent(1, () => actor.triggerEvent(`remove_leash`));
                            }
                            else {
                                actor.Properties.StopFollowing = false;
                                let followRange = (_a = readDynamicProperty(player, "party_follow_range")) !== null && _a !== void 0 ? _a : 15;
                                actor.triggerEvent(`set_leash_distance_${followRange}`);
                            }
                        }
                    },
                    {
                        type: "dropdown",
                        text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.label", with: ["\n"] },
                        defaultValueIndex: () => {
                            switch (actor.Properties.FightingPriority) {
                                case "passive":
                                    return 0;
                                case "defensive":
                                    return 1;
                                case "proactive":
                                    return 2;
                                case "aggressive":
                                    return 3;
                                default:
                                    return 0;
                            }
                        },
                        options: [
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.passive" },
                                onSelected: () => {
                                    actor.Properties.FightingPriority = "passive";
                                }
                            },
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.defensive" },
                                onSelected: () => {
                                    actor.Properties.FightingPriority = "defensive";
                                }
                            },
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.proactive" },
                                onSelected: () => {
                                    actor.Properties.FightingPriority = "proactive";
                                }
                            },
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.aggressive" },
                                onSelected: () => {
                                    actor.Properties.FightingPriority = "aggressive";
                                }
                            }
                        ]
                    },
                    {
                        type: "dropdown",
                        text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.mood", with: ["\n"] },
                        defaultValueIndex: (actor.Properties.Mood >= 5) ? actor.Properties.PreviousMood : actor.Properties.Mood,
                        options: [
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.mood.normal" },
                                onSelected: () => {
                                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                    if (actor.Properties.Mood >= 5) {
                                        actor.Properties.PreviousMood = 0;
                                    }
                                    else {
                                        actor.Properties.Mood = 0;
                                    }
                                }
                            },
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.mood.happy" },
                                onSelected: () => {
                                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                    if (actor.Properties.Mood >= 5) {
                                        actor.Properties.PreviousMood = 1;
                                    }
                                    else {
                                        actor.Properties.Mood = 1;
                                    }
                                }
                            },
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.mood.smug" },
                                onSelected: () => {
                                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                    if (actor.Properties.Mood >= 5) {
                                        actor.Properties.PreviousMood = 2;
                                    }
                                    else {
                                        actor.Properties.Mood = 2;
                                    }
                                }
                            },
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.mood.intense" },
                                onSelected: () => {
                                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                    if (actor.Properties.Mood >= 5) {
                                        actor.Properties.PreviousMood = 3;
                                    }
                                    else {
                                        actor.Properties.Mood = 3;
                                    }
                                }
                            },
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.fighting_priority.mood.piercing" },
                                onSelected: () => {
                                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                    if (actor.Properties.Mood >= 5) {
                                        actor.Properties.PreviousMood = 4;
                                    }
                                    else {
                                        actor.Properties.Mood = 4;
                                    }
                                }
                            }
                        ]
                    }
                ]
            },
            stats: {
                type: "buttons",
                buttonLabel: { translate: "ldz_240913:menu.bruce_lee.stats.button" },
                icon: "textures/ldz/240913/ui/stats",
                title: { translate: "ldz_240913:menu.bruce_lee.stats.title" },
                contents: [],
                ignoreBookmark: true,
                body: { rawtext: [
                        { translate: "ldz_240913:menu.bruce_lee.stats.martial_arts_path", with: { rawtext: [{ text: "\n" }, { translate: `ldz_240913:menu.bruce_lee.martial_arts.${actor.Properties.MartialArt}` }] } },
                        { translate: "ldz_240913:menu.bruce_lee.stats.level", with: ["\n", actor.Properties.Level.toString()] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.health", with: ["\n", actor.Properties.Health.toString()] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.experience", with: ["\n", actor.Properties.Experience.toString(), actor.Properties.ExperienceToNextLevel.toString()] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.block_chance", with: ["\n", actor.Properties.BlockChance.toString(), "\u0025\u0025"] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.dragon_fury", with: ["\n", actor.Properties.DragonFuryLevel.toString()] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.damage_dealt", with: ["\n", actor.Properties.DamageDealt.toString()] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.damage_taken", with: ["\n", actor.Properties.DamageTaken.toString()] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.attacks_blocked", with: ["\n", actor.Properties.AttacksBlocked.toString()] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.times_knocked_out", with: ["\n", actor.Properties.TimesKnockedOut.toString()] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.time_alive", with: ["\n", `${(new Date(Date.now() - actor.Properties.TimeAlive).getHours() > 0) ? `${new Date(Date.now() - actor.Properties.TimeAlive).getHours()}h ` : ""}${new Date(Date.now() - actor.Properties.TimeAlive).getMinutes()}m ${new Date(Date.now() - actor.Properties.TimeAlive).getSeconds()}s`] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.time_meditating", with: ["\n", `${(Math.floor(actor.Properties.TimeMeditating / 60) > 0) ? `${Math.floor(actor.Properties.TimeMeditating / 60)}m ` : ""}${(actor.Properties.TimeMeditating % 60)}s`] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.mobs_defeated", with: ["\n", actor.Properties.MobsDefeated.toString()] },
                        { translate: "ldz_240913:menu.bruce_lee.stats.opponents_defeated", with: ["\n", actor.Properties.OpponentsDefeated.toString()] },
                    ] }
            },
            style: {
                type: "modal",
                buttonLabel: { translate: "ldz_240913:menu.bruce_lee.style.button" },
                title: { translate: "ldz_240913:menu.bruce_lee.style.title" },
                icon: "textures/ldz/240913/ui/style",
                submitText: { translate: "ldz_240913:menu.button.confirm" },
                contents: [
                    {
                        type: "dropdown",
                        text: { translate: "ldz_240913:menu.bruce_lee.style.outfit.label" },
                        defaultValueIndex(player) {
                            let achievements = getCompletedAchievements(player);
                            mclog(achievements.map(a => a.lang), "BruceLee");
                            for (let i = 0; i < achievements.length; i++) {
                                if (actor.Properties.Outfit % 11 === achievements[i].skinIndex) {
                                    mclog(`Setting selected outfit to ${i + 1}`, "BruceLee");
                                    return i + 1;
                                }
                            }
                            return 0;
                        },
                        options: styleOptions
                    },
                    {
                        type: "dropdown",
                        text: { translate: "ldz_240913:menu.bruce_lee.style.hairstyle.label" },
                        defaultValueIndex: actor.Properties.Hairstyle,
                        options: [
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.style.hairstyle.shortest" },
                                onSelected: () => {
                                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                    if (actor.Properties.Hairstyle !== 0) {
                                        player.playSound("ldz_240913:bruce_lee.menu.change_hairstyle");
                                    }
                                    actor.Properties.Hairstyle = 0;
                                }
                            },
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.style.hairstyle.short" },
                                onSelected: () => {
                                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                    if (actor.Properties.Hairstyle !== 1) {
                                        player.playSound("ldz_240913:bruce_lee.menu.change_hairstyle");
                                    }
                                    actor.Properties.Hairstyle = 1;
                                }
                            },
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.style.hairstyle.medium" },
                                onSelected: () => {
                                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                    if (actor.Properties.Hairstyle !== 2) {
                                        player.playSound("ldz_240913:bruce_lee.menu.change_hairstyle");
                                    }
                                    actor.Properties.Hairstyle = 2;
                                }
                            },
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.style.hairstyle.longer" },
                                onSelected: () => {
                                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                    if (actor.Properties.Hairstyle !== 3) {
                                        player.playSound("ldz_240913:bruce_lee.menu.change_hairstyle");
                                    }
                                    actor.Properties.Hairstyle = 3;
                                }
                            },
                            {
                                text: { translate: "ldz_240913:menu.bruce_lee.style.hairstyle.longest" },
                                onSelected: () => {
                                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                    if (actor.Properties.Hairstyle !== 4) {
                                        player.playSound("ldz_240913:bruce_lee.menu.change_hairstyle");
                                    }
                                    actor.Properties.Hairstyle = 4;
                                }
                            }
                        ]
                    }
                ]
            },
            rest: {
                type: "buttons",
                buttonLabel: { translate: "ldz_240913:menu.bruce_lee.rest.button" },
                title: { translate: "ldz_240913:menu.bruce_lee.rest.title" },
                ignoreBookmark: true,
                icon: "textures/ldz/240913/ui/rest",
                contents: [
                    {
                        text: { translate: "ldz_240913:menu.bruce_lee.rest.meditate.button" },
                        texture: "textures/ldz/240913/ui/meditate",
                        onClick: () => {
                            if (actor.Properties.Health < actor.Properties.MaxHealth) {
                                player.playSound("ldz_240913:bruce_lee.meditate");
                                actor.Properties.Meditating = true;
                            }
                            else {
                                player.sendMessage({ translate: "ldz_240913:chat_message.meditate_full_health" });
                                player.playSound("ldz_240913:bruce_lee.menu.fail");
                            }
                        }
                    },
                    {
                        text: { translate: "ldz_240913:menu.bruce_lee.rest.retire.button" },
                        texture: "textures/ldz/240913/ui/rest",
                        onClick: () => {
                            showTakeABreakWarningMenu(actor, player);
                        }
                    }
                ]
            },
            overview: {
                type: "page_list",
                buttonLabel: { translate: "ldz_240913:menu.overview.button" },
                title: { translate: "ldz_240913:menu.overview.title" },
                icon: "textures/ldz/240913/ui/how_to_play",
                ignoreBookmark: true,
                contents: {
                    how_to_play: {
                        type: "page_list",
                        buttonLabel: { translate: "ldz_240913:menu.how_to_play.button", with: ["\n"] },
                        title: { translate: "ldz_240913:menu.how_to_play.title", with: ["\n"] },
                        icon: "textures/ldz/240913/ui/how_to_play",
                        backButtonPosition: "both",
                        ignoreBookmark: true,
                        contents: {
                            spawn_egg: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.how_to_play.spawn_egg.button", with: ["\n"] },
                                title: { translate: "ldz_240913:menu.how_to_play.spawn_egg.title", with: ["\n"] },
                                body: { translate: "ldz_240913:menu.how_to_play.spawn_egg.body", with: ["\n"] },
                                icon: "textures/ldz/240913/items/spawn_egg/bruce_lee",
                                ignoreBookmark: true,
                                contents: []
                            },
                            martial_arts_path: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.how_to_play.martial_arts_path.button", with: ["\n"] },
                                title: { translate: "ldz_240913:menu.how_to_play.martial_arts_path.title", with: ["\n"] },
                                body: { translate: "ldz_240913:menu.how_to_play.martial_arts_path.body", with: ["\n"] },
                                icon: "textures/ldz/240913/ui/martial_arts_path",
                                ignoreBookmark: true,
                                contents: []
                            },
                            health_and_xp: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.how_to_play.health_and_xp.button", with: ["\n"] },
                                title: { translate: "ldz_240913:menu.how_to_play.health_and_xp.title", with: ["\n"] },
                                body: { translate: "ldz_240913:menu.how_to_play.health_and_xp.body", with: ["\n"] },
                                icon: "textures/ldz/240913/ui/health_and_xp",
                                ignoreBookmark: true,
                                contents: []
                            },
                            meditation: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.how_to_play.meditation.button", with: ["\n"] },
                                title: { translate: "ldz_240913:menu.how_to_play.meditation.title", with: ["\n"] },
                                body: { translate: "ldz_240913:menu.how_to_play.meditation.body", with: ["\n"] },
                                icon: "textures/ldz/240913/ui/meditate",
                                ignoreBookmark: true,
                                contents: []
                            },
                            dragon_fury: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.how_to_play.dragon_fury.button", with: ["\n"] },
                                title: { translate: "ldz_240913:menu.how_to_play.dragon_fury.title", with: ["\n"] },
                                body: { translate: "ldz_240913:menu.how_to_play.dragon_fury.body", with: ["\n"] },
                                icon: "textures/ldz/240913/ui/dragon_fury",
                                ignoreBookmark: true,
                                contents: []
                            },
                            style_costumes: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.how_to_play.style_costumes.button", with: ["\n"] },
                                title: { translate: "ldz_240913:menu.how_to_play.style_costumes.title", with: ["\n"] },
                                body: { translate: "ldz_240913:menu.how_to_play.style_costumes.body", with: ["\n"] },
                                icon: "textures/ldz/240913/ui/style",
                                ignoreBookmark: true,
                                contents: []
                            },
                            dragon_crystal_shard: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.how_to_play.dragon_crystals.button", with: ["\n"] },
                                icon: "textures/ldz/240913/items/dragon_crystal_shard",
                                title: { translate: "ldz_240913:menu.how_to_play.dragon_crystals.title", with: ["\n"] },
                                body: { translate: "ldz_240913:menu.how_to_play.dragon_crystals.body", with: ["\n"] },
                                ignoreBookmark: true,
                                contents: []
                            },
                            crafting_recipes: {
                                type: "buttons",
                                buttonLabel: { translate: "ldz_240913:menu.how_to_play.crafting_recipes.button", with: ["\n"] },
                                title: { translate: "ldz_240913:menu.how_to_play.crafting_recipes.title", with: ["\n"] },
                                body: { translate: "ldz_240913:menu.how_to_play.crafting_recipes.body", with: ["\n"] },
                                icon: "textures/ldz/240913/ui/crafting_recipes",
                                ignoreBookmark: true,
                                contents: []
                            }
                        }
                    },
                    changelog: {
                        type: "buttons",
                        buttonLabel: { translate: "ldz_240913:menu.changelog.button", with: ["\n"] },
                        title: { translate: "ldz_240913:update.version.name", with: [utils.getVersionNumber()] },
                        body: { translate: "ldz_240913:menu.changelog.body", with: ["\n"] },
                        icon: "textures/ldz/240913/ui/changelog",
                        ignoreBookmark: true,
                        contents: []
                    },
                    achievements: {
                        type: "buttons",
                        buttonLabel: { translate: "ldz_240913:menu.achievements.button" },
                        title: { translate: "ldz_240913:menu.achievements.title" },
                        body: (player) => getAchievementsMenuBodyText(player),
                        icon: "textures/ldz/240913/ui/achievements",
                        ignoreBookmark: true,
                        contents: []
                    },
                    settings: {
                        type: "modal",
                        buttonLabel: { translate: "ldz_240913:menu.settings.button", with: ["\n"] },
                        title: { translate: "ldz_240913:menu.settings.title", with: ["\n"] },
                        icon: "textures/ldz/240913/ui/settings",
                        submitText: { translate: "ldz_240913:menu.button.confirm" },
                        contents: [
                            {
                                type: "slider",
                                text: { translate: "ldz_240913:menu.settings.follow_range" },
                                maximumValue: 50,
                                minimumValue: 15,
                                valueStep: 1,
                                defaultValue(player) {
                                    var _a;
                                    let value = (_a = readDynamicProperty(player, "party_follow_range")) !== null && _a !== void 0 ? _a : 40;
                                    if (!(typeof value === "number"))
                                        value = 40;
                                    return value;
                                },
                                onSubmit(player, sliderValue) {
                                    writeDynamicProperty(player, "party_follow_range", sliderValue);
                                    BruceLee.getActors(BruceLee).forEach(actor => {
                                        if (actor.isValid() && actor.Properties.OwnerId === player.id) {
                                            actor.triggerEvent(`set_leash_distance_${sliderValue}`);
                                        }
                                    });
                                },
                            },
                            {
                                type: "toggle",
                                text: { translate: "ldz_240913:menu.settings.info_panel" },
                                defaultValue(player) {
                                    return player.hasTag("show_panel");
                                },
                                onSubmit(player, toggleValue) {
                                    if (toggleValue) {
                                        player.addTag("show_panel");
                                    }
                                    else {
                                        player.removeTag("show_panel");
                                    }
                                },
                            },
                            {
                                type: "toggle",
                                text: { translate: "ldz_240913:menu.settings.low_health_warning" },
                                defaultValue(player) {
                                    var _a;
                                    let value = (_a = readDynamicProperty(player, "ldz_240913:low_health_warning")) !== null && _a !== void 0 ? _a : true;
                                    if (!(typeof value === "boolean"))
                                        value = true;
                                    return value;
                                },
                                onSubmit(player, toggleValue) {
                                    BruceLee.getActors(BruceLee).forEach(actor => {
                                        if (actor.isValid() && actor.Properties.OwnerId === player.id) {
                                            actor.Properties.HealthFlash = toggleValue ? "enabled" : "disabled";
                                        }
                                    });
                                    writeDynamicProperty(player, "ldz_240913:low_health_warning", toggleValue);
                                },
                            },
                            {
                                type: "toggle",
                                text: { translate: "ldz_240913:menu.settings.player_damage" },
                                defaultValue(player) {
                                    return !player.hasTag("no_damage_party");
                                },
                                onSubmit(player, toggleValue) {
                                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                    if (!toggleValue) {
                                        player.addTag("no_damage_party");
                                    }
                                    else {
                                        player.removeTag("no_damage_party");
                                    }
                                },
                            },
                            {
                                type: "toggle",
                                text: { translate: "ldz_240913:menu.settings.fury_hits_players" },
                                defaultValue(player) {
                                    var _a;
                                    let value = (_a = readDynamicProperty(player, "ldz_240913:fury_hits_players")) !== null && _a !== void 0 ? _a : true;
                                    if (!(typeof value === "boolean"))
                                        value = true;
                                    return value;
                                },
                                onSubmit(player, toggleValue) {
                                    writeDynamicProperty(player, "ldz_240913:fury_hits_players", toggleValue);
                                },
                            },
                            {
                                type: "toggle",
                                text: { translate: "ldz_240913:menu.settings.retaliate_against_players" },
                                defaultValue(player) {
                                    var _a;
                                    let value = (_a = readDynamicProperty(world, "ldz_240913:retaliate_against_players")) !== null && _a !== void 0 ? _a : true;
                                    if (!(typeof value === "boolean"))
                                        value = true;
                                    return value;
                                },
                                onSubmit(player, toggleValue) {
                                    writeDynamicProperty(world, "ldz_240913:retaliate_against_players", toggleValue);
                                    overworld.getEntities({ type: "ldz_240913:bruce_lee" }).forEach(entity => {
                                        if (!toggleValue)
                                            entity.addTag("ldz_240913:dont_retaliate_against_players");
                                        else
                                            entity.removeTag("ldz_240913:dont_retaliate_against_players");
                                    });
                                    nether.getEntities({ type: "ldz_240913:bruce_lee" }).forEach(entity => {
                                        if (!toggleValue)
                                            entity.addTag("ldz_240913:dont_retaliate_against_players");
                                        else
                                            entity.removeTag("ldz_240913:dont_retaliate_against_players");
                                    });
                                    end.getEntities({ type: "ldz_240913:bruce_lee" }).forEach(entity => {
                                        if (!toggleValue)
                                            entity.addTag("ldz_240913:dont_retaliate_against_players");
                                        else
                                            entity.removeTag("ldz_240913:dont_retaliate_against_players");
                                    });
                                },
                            },
                            {
                                type: "toggle",
                                text: { translate: "ldz_240913:menu.settings.send_death_messages" },
                                defaultValue(player) {
                                    var _a;
                                    let value = (_a = readDynamicProperty(player, "ldz_240913:send_death_messages")) !== null && _a !== void 0 ? _a : true;
                                    if (!(typeof value === "boolean"))
                                        value = true;
                                    return value;
                                },
                                onSubmit(player, toggleValue) {
                                    writeDynamicProperty(player, "ldz_240913:send_death_messages", toggleValue);
                                },
                            },
                            {
                                type: "toggle",
                                text: { translate: "ldz_240913:menu.settings.disable_idle_lines" },
                                defaultValue(player) {
                                    return !player.hasTag("disable_idle_lines");
                                },
                                onSubmit(player, toggleValue) {
                                    if (!toggleValue) {
                                        player.addTag("disable_idle_lines");
                                    }
                                    else {
                                        player.removeTag("disable_idle_lines");
                                        BruceLee.getActors(BruceLee).forEach(actor => {
                                            if (actor.isValid() && actor.Properties.OwnerId === player.id) {
                                                actor.IdleLines.startTimer();
                                            }
                                        });
                                    }
                                },
                            },
                            {
                                type: "slider",
                                text: { translate: "ldz_240913:menu.settings.volume.bruce_lee" },
                                minimumValue: 0,
                                maximumValue: 20,
                                valueStep: 1,
                                defaultValue(player) {
                                    var _a;
                                    let value = (_a = readDynamicProperty(player, BruceLeeSoundPlayer.VOLUME_DYNAMIC_PROPERTY)) !== null && _a !== void 0 ? _a : 10;
                                    if (!(typeof value === "number"))
                                        value = 10;
                                    return value;
                                },
                                onSubmit(player, sliderValue) {
                                    writeDynamicProperty(player, BruceLeeSoundPlayer.VOLUME_DYNAMIC_PROPERTY, sliderValue);
                                },
                            },
                            {
                                type: "slider",
                                text: { translate: "ldz_240913:menu.settings.volume.opponents" },
                                minimumValue: 0,
                                maximumValue: 20,
                                valueStep: 1,
                                defaultValue(player) {
                                    var _a;
                                    let value = (_a = readDynamicProperty(player, OpponentSoundPlayer.VOLUME_DYNAMIC_PROPERTY)) !== null && _a !== void 0 ? _a : 10;
                                    if (!(typeof value === "number"))
                                        value = 10;
                                    return value;
                                },
                                onSubmit(player, sliderValue) {
                                    writeDynamicProperty(player, OpponentSoundPlayer.VOLUME_DYNAMIC_PROPERTY, sliderValue);
                                },
                            },
                            {
                                type: "toggle",
                                text: { translate: "ldz_240913:menu.settings.opponents_during_day" },
                                defaultValue() {
                                    var _a;
                                    let value = (_a = readDynamicProperty(world, "ldz_240913:opponents_during_day")) !== null && _a !== void 0 ? _a : true;
                                    if (!(typeof value === "boolean"))
                                        value = true;
                                    return value;
                                },
                                onSubmit(player, toggleValue) {
                                    writeDynamicProperty(world, "ldz_240913:opponents_during_day", toggleValue);
                                },
                            },
                        ]
                    },
                    credits: {
                        type: "buttons",
                        buttonLabel: { translate: "ldz_240913:menu.credits.button", with: ["\n"] },
                        title: { translate: "ldz_240913:update.version.name", with: [utils.getVersionNumber()] },
                        body: { translate: "ldz_240913:menu.credits.body", with: ["\n"] },
                        icon: "textures/ldz/240913/ui/credits",
                        ignoreBookmark: true,
                        contents: []
                    }
                }
            }
        }
    }, {
        bookmarkPropertyKey: "ldz_240913:bruce_lee.menu.bookmark",
        defaultBackIcon: "textures/ldz/240913/ui/back",
        defaultSubmitText: { translate: "ldz_240913:menu.button.confirm" },
    });
    let isOpen = true;
    mclog(` [${system.currentTick / 20}] Showing menu to player ${player.name}`, "BruceLee");
    menu.showMenuToPlayer(player, path).then(() => {
        isOpen = false;
        mclog("Menu closed", "BruceLee");
    });
    waitForLoopEvent(5, () => {
        var _a, _b;
        if (isOpen && ((_b = (_a = actor.Owner) === null || _a === void 0 ? void 0 : _a.location.distance(actor.location)) !== null && _b !== void 0 ? _b : 0) < 6) {
            actor.addEffect("minecraft:slowness", 5, { amplifier: 100, showParticles: false });
        }
        return !isOpen;
    });
    return;
}
function showPathChosenMenu(actor, player) {
    const menu = new MenuBuilder({
        type: "buttons",
        title: { translate: "ldz_240913:menu.bruce_lee.martial_arts.title" },
        body: { translate: "ldz_240913:menu.bruce_lee.martial_arts.path_chosen", with: { rawtext: [{ "text": "\n" }, { "translate": `ldz_240913:menu.bruce_lee.martial_arts.${actor.Properties.MartialArt}` }] } },
        contents: [
            {
                text: { translate: "ldz_240913:menu.button.back" },
                texture: "textures/ldz/240913/ui/back",
                onClick: () => {
                    showMenuToPlayer(actor, player);
                }
            }
        ]
    });
    let isOpen = true;
    menu.showMenuToPlayer(player).then(() => {
        isOpen = false;
        mclog("Menu closed", "BruceLee");
    });
    waitForLoopEvent(5, () => {
        if (isOpen) {
            actor.addEffect("minecraft:slowness", 5, { amplifier: 100, showParticles: false });
        }
        return !isOpen;
    });
    actor.Properties.CanChargeDragonFury = true;
}
function showNotEnoughSkillPointsMenu(actor, player, cost, currentValue, upgrade, maxValue) {
    let body = { translate: "ldz_240913:menu.bruce_lee.martial_arts.not_enough_skill_points", with: { rawtext: [{ "text": "\n" }, { "text": cost }, { "text": `${actor.Properties.SkillPoints}` }, { "text": currentValue }, { translate: upgrade }] } };
    if (maxValue && cost === "1") {
        body = { translate: "ldz_240913:menu.bruce_lee.martial_arts.not_enough_skill_point_max", with: { rawtext: [{ "text": "\n" }, { "text": cost }, { "text": `${actor.Properties.SkillPoints}` }, { "text": currentValue }, { "text": maxValue }, { translate: upgrade }] } };
    }
    else if (maxValue) {
        body = { translate: "ldz_240913:menu.bruce_lee.martial_arts.not_enough_skill_points_max", with: { rawtext: [{ "text": "\n" }, { "text": cost }, { "text": `${actor.Properties.SkillPoints}` }, { "text": currentValue }, { "text": maxValue }, { translate: upgrade }] } };
    }
    else if (cost === "1") {
        body = { translate: "ldz_240913:menu.bruce_lee.martial_arts.not_enough_skill_point", with: { rawtext: [{ "text": "\n" }, { "text": cost }, { "text": `${actor.Properties.SkillPoints}` }, { "text": currentValue }, { translate: upgrade }] } };
    }
    const menu = new MenuBuilder({
        type: "buttons",
        title: { translate: "ldz_240913:menu.bruce_lee.martial_arts.title" },
        body: body,
        contents: [
            {
                text: { translate: "ldz_240913:menu.button.back" },
                texture: "textures/ldz/240913/ui/back",
                onClick: () => {
                    showMenuToPlayer(actor, player, "root/martialArts");
                }
            }
        ]
    });
    let isOpen = true;
    menu.showMenuToPlayer(player).then(() => {
        isOpen = false;
        mclog("Menu closed", "BruceLee");
    });
    waitForLoopEvent(5, () => {
        if (isOpen) {
            actor.addEffect("minecraft:slowness", 5, { amplifier: 100, showParticles: false });
        }
        return !isOpen;
    });
}
function showTakeABreakWarningMenu(actor, player) {
    const menu = new MenuBuilder({
        type: "buttons",
        title: { translate: "ldz_240913:menu.bruce_lee.rest.title" },
        body: { translate: "ldz_240913:menu.bruce_lee.rest.retire.body", with: ["\n"] },
        contents: [
            {
                text: { translate: "ldz_240913:menu.button.confirm" },
                texture: "textures/ui/confirm",
                onClick: () => {
                    player.playSound("ldz_240913:bruce_lee.menu.confirm");
                    actor.takeABreak();
                }
            },
            {
                text: { translate: "ldz_240913:menu.button.back" },
                texture: "textures/ldz/240913/ui/back",
                onClick: () => {
                    showMenuToPlayer(actor, player);
                }
            }
        ]
    });
    let isOpen = true;
    menu.showMenuToPlayer(player).then(() => {
        isOpen = false;
        mclog("Menu closed", "BruceLee");
    });
    waitForLoopEvent(5, () => {
        if (isOpen) {
            actor.addEffect("minecraft:slowness", 5, { amplifier: 100, showParticles: false });
        }
        return !isOpen;
    });
}
