import { world } from "@minecraft/server";
import { ActorComponent, mclog, readDynamicProperty, waitForTicks, writeDynamicProperty } from "../utils/index";
const STORAGE_PROP = "ldz_240913:bruce_lee_storage";
export class PersistantActorStorage extends ActorComponent {
    constructor(actor, options) {
        var _a, _b;
        super(actor, options);
        this.storage = {};
        this.storageKey = (_a = readDynamicProperty(this.actor, "ldz_240913:storage_key")) !== null && _a !== void 0 ? _a : options.storageKey;
        this.storageKey !== this.actor.id ? writeDynamicProperty(this.actor, "ldz_240913:storage_key", this.storageKey) : "";
        this.storage = (_b = readDynamicProperty(world, STORAGE_PROP)) !== null && _b !== void 0 ? _b : {};
        mclog(`Currently storing ${Object.keys(this.storage).length} Bruce Lees`, "PersistantActorStorage");
        if (this.storage[this.storageKey] === undefined) {
            mclog(`No storage found for ${this.storageKey}`, "PersistantActorStorage");
            this.storage = Object.assign(Object.assign({}, this.storage), { [this.storageKey]: {} });
        }
        this.storage[this.storageKey]["LinkedEntity"] = this.actor.id;
        writeDynamicProperty(world, STORAGE_PROP, this.storage);
        waitForTicks(1).then(() => {
            if (!this.actor.isValid())
                return;
            this.options.listenTo.forEach(property => {
                if (this.storage[this.storageKey][property] !== undefined) {
                    mclog(`Setting ${property} to ${this.storage[this.storageKey][property]}`, "PersistantActorStorage");
                    this.actor.Properties.setProperty(property, this.storage[this.storageKey][property]);
                }
            });
        });
    }
    onPropertyChange(property, value) {
        var _a;
        if (this.options.listenTo.includes(property)) {
            this.storage = (_a = readDynamicProperty(world, STORAGE_PROP)) !== null && _a !== void 0 ? _a : {};
            this.storage[this.storageKey][property] = value;
            writeDynamicProperty(world, STORAGE_PROP, this.storage);
        }
    }
    getStorageForActor(id) {
        return this.storage[id];
    }
    useStorageFromActor(id) {
        let storage = this.storage[id];
        mclog(`Currently storing ${Object.keys(this.storage).length} Bruce Lees`, "PersistantActorStorage");
        if (storage === undefined) {
            mclog(`No storage found for ${id}`, "PersistantActorStorage");
            return;
        }
        if (storage["Retired"]) {
            storage["Retired"] = false;
        }
        delete this.storage[this.storageKey];
        writeDynamicProperty(world, STORAGE_PROP, this.storage);
        this.actor.removeActorComponent(PersistantActorStorage);
        this.actor.addActorComponent(PersistantActorStorage, {
            listenTo: this.options.listenTo,
            storageKey: id
        });
    }
}
