import * as utils from "./utils/index";
import { ItemStack, system, world } from "@minecraft/server";
import { MenuBuilder, SafetyCheckedPlayer, readDynamicProperty, writeDynamicProperty, getAllPlayersSafe, clearDynamicProperty, overworld, nether, end, MenuResult, mclog } from "./utils/index";
import { BRUCE_LEE_OUTFITS, BruceLee } from "./bruce_lee/bruce_lee";
import { getAchievementsMenuBodyText } from "./achievements";
import { BruceLeeSoundPlayer } from "./bruce_lee/sound_player";
import { OpponentSoundPlayer } from "./opponents/sound_player";
import { Outline } from "./outline";
import * as quote from "./scrolls/quotes";
import { BackupSystem } from "./bruce_lee/backup";
export class GuideBook {
    constructor(owner) {
        this.onInteractBinding = this.onInteract.bind(this);
        this.owner = owner;
        world.afterEvents.itemUse.subscribe(this.onInteractBinding);
    }
    static initializeGuidebook() {
        world.afterEvents.playerSpawn.subscribe(event => {
            if (event.initialSpawn) {
                new GuideBook(new SafetyCheckedPlayer(event.player));
                if (readDynamicProperty(event.player, "ldz_240913:received_guide_book"))
                    return;
                event.player.dimension.spawnItem(new ItemStack("ldz_240913:guide_book"), event.player.location);
                event.player.dimension.spawnItem(new ItemStack("ldz_240913:bruce_lee_spawn_egg"), event.player.location);
                writeDynamicProperty(event.player, "ldz_240913:received_guide_book", true);
            }
        });
        getAllPlayersSafe().forEach(player => {
            new GuideBook(player);
        });
        system.afterEvents.scriptEventReceive.subscribe(event => {
            if (event.id === "ldz_240913:reset_recieved_tablet" && event.sourceEntity) {
                clearDynamicProperty(event.sourceEntity, "ldz_240913:received_guide_book");
            }
            else if (event.id === "ldz_240913:reset_storage") {
                clearDynamicProperty(world, "ldz_240913:bruce_lee_storage");
            }
        });
    }
    onInteract(event) {
        var _a;
        if (event.itemStack.typeId === "ldz_240913:guide_book" && event.source.id === this.owner.id) {
            if (!Outline.getTarget(this.owner)) {
                this.showMenu();
            }
            else {
                (_a = Outline.getTarget(this.owner)) === null || _a === void 0 ? void 0 : _a.onGuideBookInteract(this.owner);
            }
        }
    }
    showMenu() {
        //const player_version = readDynamicProperty<string>(this.owner, "version") ?? "1.0.0"
        const menu = new MenuBuilder({
            type: "page_list",
            title: { translate: "ldz_240913:menu.overview.title" },
            icon: "textures/ldz/240913/ui/how_to_play",
            contents: {
                how_to_play: {
                    type: "page_list",
                    buttonLabel: { translate: "ldz_240913:menu.how_to_play.button", with: ["\n"] },
                    title: { translate: "ldz_240913:menu.how_to_play.title", with: ["\n"] },
                    icon: "textures/ldz/240913/ui/how_to_play",
                    backIcon: "textures/ldz/240913/ui/back",
                    backButtonPosition: "both",
                    contents: {
                        spawn_egg: {
                            type: "buttons",
                            buttonLabel: { translate: "ldz_240913:menu.how_to_play.spawn_egg.button", with: ["\n"] },
                            title: { translate: "ldz_240913:menu.how_to_play.spawn_egg.title", with: ["\n"] },
                            body: { translate: "ldz_240913:menu.how_to_play.spawn_egg.body", with: ["\n"] },
                            icon: "textures/ldz/240913/items/spawn_egg/bruce_lee",
                            contents: []
                        },
                        martial_arts_path: {
                            type: "buttons",
                            buttonLabel: { translate: "ldz_240913:menu.how_to_play.martial_arts_path.button", with: ["\n"] },
                            title: { translate: "ldz_240913:menu.how_to_play.martial_arts_path.title", with: ["\n"] },
                            body: { translate: "ldz_240913:menu.how_to_play.martial_arts_path.body", with: ["\n"] },
                            icon: "textures/ldz/240913/ui/martial_arts_path",
                            contents: []
                        },
                        health_and_xp: {
                            type: "buttons",
                            buttonLabel: { translate: "ldz_240913:menu.how_to_play.health_and_xp.button", with: ["\n"] },
                            title: { translate: "ldz_240913:menu.how_to_play.health_and_xp.title", with: ["\n"] },
                            body: { translate: "ldz_240913:menu.how_to_play.health_and_xp.body", with: ["\n"] },
                            icon: "textures/ldz/240913/ui/health_and_xp",
                            contents: []
                        },
                        meditation: {
                            type: "buttons",
                            buttonLabel: { translate: "ldz_240913:menu.how_to_play.meditation.button", with: ["\n"] },
                            title: { translate: "ldz_240913:menu.how_to_play.meditation.title", with: ["\n"] },
                            body: { translate: "ldz_240913:menu.how_to_play.meditation.body", with: ["\n"] },
                            icon: "textures/ldz/240913/ui/meditate",
                            contents: []
                        },
                        dragon_fury: {
                            type: "buttons",
                            buttonLabel: { translate: "ldz_240913:menu.how_to_play.dragon_fury.button", with: ["\n"] },
                            title: { translate: "ldz_240913:menu.how_to_play.dragon_fury.title", with: ["\n"] },
                            body: { translate: "ldz_240913:menu.how_to_play.dragon_fury.body", with: ["\n"] },
                            icon: "textures/ldz/240913/ui/dragon_fury",
                            contents: []
                        },
                        style_costumes: {
                            type: "buttons",
                            buttonLabel: { translate: "ldz_240913:menu.how_to_play.style_costumes.button", with: ["\n"] },
                            icon: "textures/ldz/240913/ui/style",
                            title: { translate: "ldz_240913:menu.how_to_play.style_costumes.title", with: ["\n"] },
                            body: { translate: "ldz_240913:menu.how_to_play.style_costumes.body", with: ["\n"] },
                            contents: []
                        },
                        dragon_crystal_shard: {
                            type: "buttons",
                            buttonLabel: { translate: "ldz_240913:menu.how_to_play.dragon_crystals.button", with: ["\n"] },
                            icon: "textures/ldz/240913/items/dragon_crystal_shard",
                            title: { translate: "ldz_240913:menu.how_to_play.dragon_crystals.title", with: ["\n"] },
                            body: { translate: "ldz_240913:menu.how_to_play.dragon_crystals.body", with: ["\n"] },
                            contents: []
                        },
                        knowledge_scrolls: {
                            type: "buttons",
                            buttonLabel: { translate: "ldz_240913:menu.how_to_play.knowledge_scrolls.button", with: ["\n"] },
                            icon: "textures/ldz/240913/items/scroll",
                            title: { translate: "ldz_240913:menu.how_to_play.knowledge_scrolls.title", with: ["\n"] },
                            body: { translate: "ldz_240913:menu.how_to_play.knowledge_scrolls.body", with: ["\n"] },
                            contents: []
                        },
                        crafting_recipes: {
                            type: "buttons",
                            buttonLabel: { translate: "ldz_240913:menu.how_to_play.crafting_recipes.button", with: ["\n"] },
                            icon: "textures/ldz/240913/ui/crafting_recipes",
                            title: { translate: "ldz_240913:menu.how_to_play.crafting_recipes.title", with: ["\n"] },
                            body: { translate: "ldz_240913:menu.how_to_play.crafting_recipes.body", with: ["\n"] },
                            contents: []
                        }
                    }
                },
                changelog: {
                    type: "buttons",
                    buttonLabel: { translate: "ldz_240913:menu.changelog.button", with: ["\n"] },
                    icon: "textures/ldz/240913/ui/changelog",
                    backIcon: "textures/ldz/240913/ui/back",
                    title: { translate: "ldz_240913:update.version.name", with: [utils.getVersionNumber()] },
                    body: { translate: "ldz_240913:menu.changelog.body", with: ["\n"] },
                    contents: []
                },
                achievements: {
                    type: "buttons",
                    buttonLabel: { translate: "ldz_240913:menu.achievements.button", with: ["\n"] },
                    icon: "textures/ldz/240913/ui/achievements",
                    backIcon: "textures/ldz/240913/ui/back",
                    title: { translate: "ldz_240913:menu.achievements.title" },
                    body: (player) => getAchievementsMenuBodyText(player),
                    contents: []
                },
                call_bruce_lees: {
                    type: "buttons",
                    buttonLabel: { translate: "ldz_240913:menu.call_bruce_lees.button", with: ["\n"] },
                    icon: "textures/ldz/240913/ui/stop_meditating",
                    backIcon: "textures/ldz/240913/ui/back",
                    title: { translate: "ldz_240913:menu.call_bruce_lees.title", with: ["\n"] },
                    body: { translate: "ldz_240913:menu.call_bruce_lees.body", with: ["\n"] },
                    contents: this.generateBruceLeeButtons()
                },
                quotes: {
                    type: "page_list",
                    buttonLabel: { translate: "ldz_240913:menu.quotes.button", with: ["\n"] },
                    icon: "textures/ldz/240913/items/scroll",
                    backIcon: "textures/ldz/240913/ui/back",
                    backButtonPosition: "both",
                    title: { translate: "ldz_240913:menu.quotes.title", with: ["\n"] },
                    body: { translate: "ldz_240913:menu.quotes.body", with: ["\n"] },
                    onPageOpen(player) {
                        quote.stopAllQuoteSounds(player);
                    },
                    contents: quote.generateQuoteButtons(this.owner),
                },
                settings: {
                    type: "modal",
                    buttonLabel: { translate: "ldz_240913:menu.settings.button", with: ["\n"] },
                    icon: "textures/ldz/240913/ui/settings",
                    title: { translate: "ldz_240913:menu.settings.title", with: ["\n"] },
                    submitText: { translate: "ldz_240913:menu.button.confirm" },
                    contents: [
                        {
                            type: "slider",
                            text: { translate: "ldz_240913:menu.settings.follow_range" },
                            maximumValue: 50,
                            minimumValue: 15,
                            valueStep: 1,
                            defaultValue(player) {
                                var _a;
                                let value = (_a = readDynamicProperty(player, "party_follow_range")) !== null && _a !== void 0 ? _a : 15;
                                if (!(typeof value === "number"))
                                    value = 15;
                                return value;
                            },
                            onSubmit(player, sliderValue) {
                                player.playSound("ldz_240913:bruce_lee.menu.confirm");
                                writeDynamicProperty(player, "party_follow_range", sliderValue);
                                BruceLee.getActors(BruceLee).forEach(actor => {
                                    if (actor.isValid() && actor.Properties.OwnerId === player.id && actor.Properties.StopFollowing === false) {
                                        actor.triggerEvent(`set_leash_distance_${sliderValue}`);
                                    }
                                });
                            },
                        },
                        {
                            type: "toggle",
                            text: { translate: "ldz_240913:menu.settings.info_panel" },
                            defaultValue(player) {
                                return player.hasTag("show_panel");
                            },
                            onSubmit(player, toggleValue) {
                                if (toggleValue) {
                                    player.addTag("show_panel");
                                }
                                else {
                                    player.removeTag("show_panel");
                                }
                            },
                        },
                        {
                            type: "toggle",
                            text: { translate: "ldz_240913:menu.settings.low_health_warning" },
                            defaultValue(player) {
                                var _a;
                                let value = (_a = readDynamicProperty(player, "ldz_240913:low_health_warning")) !== null && _a !== void 0 ? _a : true;
                                if (!(typeof value === "boolean"))
                                    value = true;
                                return value;
                            },
                            onSubmit(player, toggleValue) {
                                BruceLee.getActors(BruceLee).forEach(actor => {
                                    if (actor.isValid() && actor.Properties.OwnerId === player.id) {
                                        actor.Properties.HealthFlash = toggleValue ? "enabled" : "disabled";
                                    }
                                });
                                writeDynamicProperty(player, "ldz_240913:low_health_warning", toggleValue);
                            },
                        },
                        {
                            type: "toggle",
                            text: { translate: "ldz_240913:menu.settings.player_damage" },
                            defaultValue(player) {
                                return !player.hasTag("no_damage_party");
                            },
                            onSubmit(player, toggleValue) {
                                if (!toggleValue) {
                                    player.addTag("no_damage_party");
                                }
                                else {
                                    player.removeTag("no_damage_party");
                                }
                            },
                        },
                        {
                            type: "toggle",
                            text: { translate: "ldz_240913:menu.settings.fury_hits_players" },
                            defaultValue(player) {
                                var _a;
                                let value = (_a = readDynamicProperty(player, "ldz_240913:fury_hits_players")) !== null && _a !== void 0 ? _a : true;
                                if (!(typeof value === "boolean"))
                                    value = true;
                                return value;
                            },
                            onSubmit(player, toggleValue) {
                                writeDynamicProperty(player, "ldz_240913:fury_hits_players", toggleValue);
                            },
                        },
                        {
                            type: "toggle",
                            text: { translate: "ldz_240913:menu.settings.retaliate_against_players" },
                            defaultValue(player) {
                                var _a;
                                let value = (_a = readDynamicProperty(world, "ldz_240913:retaliate_against_players")) !== null && _a !== void 0 ? _a : true;
                                if (!(typeof value === "boolean"))
                                    value = true;
                                return value;
                            },
                            onSubmit(player, toggleValue) {
                                writeDynamicProperty(world, "ldz_240913:retaliate_against_players", toggleValue);
                                overworld.getEntities({ type: "ldz_240913:bruce_lee" }).forEach(entity => {
                                    if (!toggleValue)
                                        entity.addTag("dont_retaliate_against_players");
                                    else
                                        entity.removeTag("dont_retaliate_against_players");
                                });
                                nether.getEntities({ type: "ldz_240913:bruce_lee" }).forEach(entity => {
                                    if (!toggleValue)
                                        entity.addTag("dont_retaliate_against_players");
                                    else
                                        entity.removeTag("dont_retaliate_against_players");
                                });
                                end.getEntities({ type: "ldz_240913:bruce_lee" }).forEach(entity => {
                                    if (!toggleValue)
                                        entity.addTag("dont_retaliate_against_players");
                                    else
                                        entity.removeTag("dont_retaliate_against_players");
                                });
                            },
                        },
                        {
                            type: "toggle",
                            text: { translate: "ldz_240913:menu.settings.send_death_messages" },
                            defaultValue(player) {
                                var _a;
                                let value = (_a = readDynamicProperty(player, "ldz_240913:send_death_messages")) !== null && _a !== void 0 ? _a : true;
                                if (!(typeof value === "boolean"))
                                    value = true;
                                return value;
                            },
                            onSubmit(player, toggleValue) {
                                writeDynamicProperty(player, "ldz_240913:send_death_messages", toggleValue);
                            },
                        },
                        {
                            type: "toggle",
                            text: { translate: "ldz_240913:menu.settings.disable_idle_lines" },
                            defaultValue(player) {
                                return !player.hasTag("disable_idle_lines");
                            },
                            onSubmit(player, toggleValue) {
                                if (!toggleValue) {
                                    player.addTag("disable_idle_lines");
                                }
                                else {
                                    player.removeTag("disable_idle_lines");
                                }
                            },
                        },
                        {
                            type: "slider",
                            text: { translate: "ldz_240913:menu.settings.volume.bruce_lee" },
                            minimumValue: 0,
                            maximumValue: 20,
                            valueStep: 1,
                            defaultValue(player) {
                                var _a;
                                let value = (_a = readDynamicProperty(player, BruceLeeSoundPlayer.VOLUME_DYNAMIC_PROPERTY)) !== null && _a !== void 0 ? _a : 10;
                                if (!(typeof value === "number"))
                                    value = 10;
                                return value;
                            },
                            onSubmit(player, sliderValue) {
                                writeDynamicProperty(player, BruceLeeSoundPlayer.VOLUME_DYNAMIC_PROPERTY, sliderValue);
                            },
                        },
                        {
                            type: "slider",
                            text: { translate: "ldz_240913:menu.settings.volume.opponents" },
                            minimumValue: 0,
                            maximumValue: 20,
                            valueStep: 1,
                            defaultValue(player) {
                                var _a;
                                let value = (_a = readDynamicProperty(player, OpponentSoundPlayer.VOLUME_DYNAMIC_PROPERTY)) !== null && _a !== void 0 ? _a : 10;
                                if (!(typeof value === "number"))
                                    value = 10;
                                return value;
                            },
                            onSubmit(player, sliderValue) {
                                writeDynamicProperty(player, OpponentSoundPlayer.VOLUME_DYNAMIC_PROPERTY, sliderValue);
                            },
                        },
                        {
                            type: "toggle",
                            text: { translate: "ldz_240913:menu.settings.opponents_during_day" },
                            defaultValue(player) {
                                var _a;
                                let value = (_a = readDynamicProperty(world, "ldz_240913:opponents_during_day")) !== null && _a !== void 0 ? _a : true;
                                if (!(typeof value === "boolean"))
                                    value = true;
                                return value;
                            },
                            onSubmit(player, toggleValue) {
                                writeDynamicProperty(world, "ldz_240913:opponents_during_day", toggleValue);
                            },
                        },
                    ]
                },
                credits: {
                    type: "buttons",
                    buttonLabel: { translate: "ldz_240913:menu.credits.button", with: ["\n"] },
                    icon: "textures/ldz/240913/ui/credits",
                    backIcon: "textures/ldz/240913/ui/back",
                    title: { translate: "ldz_240913:update.version.name", with: [utils.getVersionNumber()] },
                    body: { translate: "ldz_240913:menu.credits.body", with: ["\n"] },
                    contents: []
                },
            }
        }, {
            defaultBackIcon: "textures/ldz/240913/ui/back",
        });
        menu.showMenuToPlayer(this.owner).then(result => { if (result === MenuResult.UserClosed)
            quote.stopAllQuoteSounds(this.owner); });
    }
    generateBruceLeeButtons() {
        let dict = BackupSystem.BackupRegistry;
        let bruceLees = Object.keys(dict);
        mclog(`Bruce Lee buttons: ${JSON.stringify(dict)}`);
        return bruceLees.filter(id => dict[id] == this.owner.id).filter(id => BackupSystem.PropertyRegistry(id).Retired === false).map(id => {
            var _a, _b, _c;
            let storage = BackupSystem.PropertyRegistry(id);
            return {
                text: { translate: "ldz_240913:menu.call_bruce_lees.bruce_lee.button", with: { rawtext: [{ text: "\n" }, { text: (_b = (_a = storage.Level) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : "1" }, { translate: `ldz_240913:menu.bruce_lee.martial_arts.${(_c = storage.MartialArt) !== null && _c !== void 0 ? _c : "unarmed"}` }] } },
                texture: this.getIconFilepath(typeof storage.Outfit === 'number' ? storage.Outfit : 0, typeof storage.Level === 'number' ? storage.Level : 1),
                onClick: (player) => {
                    let actor = BruceLee.getActor(id, BruceLee);
                    player.playSound("ldz_240913:entity.teleport.in");
                    if (actor) {
                        actor.teleport(player.location, { dimension: player.dimension });
                    }
                    else {
                        BackupSystem.loadFromBackup(id, player.dimension, player.location).then(actor => {
                            if (actor) {
                                actor.Properties.OwnerId = player.id;
                            }
                        });
                    }
                }
            };
        });
    }
    getIconFilepath(outfit, level) {
        let localOutfit = outfit % BRUCE_LEE_OUTFITS.length;
        const stringPath = `textures/ldz/240913/ui/call/${BRUCE_LEE_OUTFITS[localOutfit]}_${level}`;
        return stringPath;
    }
}
GuideBook.initializeGuidebook();
