import { EntityDamageCause, system } from "@minecraft/server";
import { ActorComponent, clamp, getRandomInteger, SafetyCheckedEntity, waitForEvent } from "../utils/index";
import { UNARMED_OPPONENT_ATTACKS } from "./opponent_attacks";
export class OpponentAttack extends ActorComponent {
    constructor(actor, options = UNARMED_OPPONENT_ATTACKS) {
        super(actor, options);
        this.eventBinding = this.onEventRecieved.bind(this);
        this.validTargets = [
            { type: "player" },
            { families: ["mob"] }
        ];
        this.nextHitTime = 0;
    }
    onComponentAdded() {
        system.afterEvents.scriptEventReceive.subscribe(this.eventBinding);
    }
    onComponentRemoved() {
        system.afterEvents.scriptEventReceive.unsubscribe(this.eventBinding);
    }
    addAttack(attack) {
        this.options.attacks.push(attack);
    }
    removeAttack(attack) {
        const index = this.options.attacks.indexOf(attack);
        if (index !== -1) {
            this.options.attacks.splice(index, 1);
        }
    }
    onEventRecieved(event) {
        var _a;
        if (this.nextHitTime >= system.currentTick)
            return;
        if (event.id === "ldz_240913:attack" && event.message === "attacker" && ((_a = event.sourceEntity) === null || _a === void 0 ? void 0 : _a.id) === this.actor.id) {
            const subEvent = (subEvent) => {
                if (subEvent.id === "ldz_240913:attack" && subEvent.message === "victim") {
                    const victim = subEvent.sourceEntity;
                    if (victim !== undefined && victim.id !== this.actor.id) {
                        this.performAttack(new SafetyCheckedEntity(victim));
                    }
                }
            };
            system.afterEvents.scriptEventReceive.subscribe(subEvent);
            waitForEvent(1, () => {
                system.afterEvents.scriptEventReceive.unsubscribe(subEvent);
            });
        }
    }
    performAttack(victim) {
        var _a;
        const attackIndex = clamp(getRandomInteger(0, this.options.attacks.length - 1), 0, this.options.attacks.length - 1);
        const attack = this.options.attacks[attackIndex];
        const distance = this.actor.location.distance(victim.location);
        // mclog(`Performing attack ${attackIndex} with distance ${distance}`, "Opponent")
        if (distance <= attack.range) {
            this.nextHitTime = system.currentTick + (attack.duration.endTime * 20) + (attack.cooldown * 20);
            this.actor.addEffect("slowness", attack.duration.endTime * 20, { amplifier: 255, showParticles: false });
            // mclog(`Playing animation animation.${this.actor.typeId.replace(":", ".")}.${attack.animation}`, "Opponent")
            this.actor.playAnimation(`animation.${this.actor.typeId.replace(":", ".")}.${attack.animation}`);
            if (attack.soundEvent)
                (_a = this.actor.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound(attack.soundEvent);
            attack.duration.damageTimes.forEach(time => {
                waitForEvent(time * 20, () => {
                    if (this.actor.isValid() && victim.isValid()) {
                        victim.applyDamage(attack.damage, { cause: EntityDamageCause.entityAttack, damagingEntity: this.actor });
                        waitForEvent(2, () => {
                            var _a;
                            if (victim.isValid() && victim.typeId === "ldz_240913:bruce_lee") {
                                const health = victim.getProperty('ldz_240913:health');
                                if (health === 0) {
                                    this.actor.attemptTaunt();
                                }
                            }
                            else if (victim.isValid() && victim.typeId === "minecraft:player") {
                                const health = (_a = victim.Health) === null || _a === void 0 ? void 0 : _a.currentValue;
                                if (health === 0) {
                                    this.actor.attemptTaunt();
                                }
                            }
                            else if (!victim.isValid() || !victim) {
                                this.actor.attemptTaunt();
                            }
                        });
                    }
                });
            });
            waitForEvent(attack.duration.endTime * 20, () => {
                if (this.actor.isValid()) {
                    this.actor.playAnimation(`animation.${this.actor.typeId.replace(":", ".")}.stop_attack`);
                }
            });
        }
    }
}
