import { EntityDamageCause, GameMode, world } from "@minecraft/server";
import * as utils from "../utils/index";
import { OpponentAttack } from "./attack";
import { DUAL_SWORDS_OPPONENT_ATTACKS, FRYING_PAN_OPPONENT_ATTACKS, KNIFE_OPPONENT_ATTACKS, MOP_OPPONENT_ATTACKS, POLE_OPPONENT_ATTACKS, SWORD_OPPONENT_ATTACKS, UNARMED_OPPONENT_ATTACKS } from "./opponent_attacks";
import { OPPONENT_SOUNDS, OpponentSoundPlayer } from "./sound_player";
const LOGTAG = "Opponent";
const WEAPONS = ["unarmed", "sword", "knife", "dual_swords", "pole", "mop", "frying_pan"];
const ATTACKS = [UNARMED_OPPONENT_ATTACKS, SWORD_OPPONENT_ATTACKS, KNIFE_OPPONENT_ATTACKS, DUAL_SWORDS_OPPONENT_ATTACKS, POLE_OPPONENT_ATTACKS, MOP_OPPONENT_ATTACKS, FRYING_PAN_OPPONENT_ATTACKS];
const WEIGHTS = [10, 1, 1, 1, 1, 1, 1];
const TAUNT_RNG_RANGE = [1, 3];
const TAUNT_DELAY_RANGE = [0, 1];
const SKINS_COUNT = 16;
const cumulativeWeights = WEIGHTS.reduce((acc, weight, index) => {
    if (index === 0) {
        acc.push(weight);
    }
    else {
        acc.push(weight + acc[index - 1]);
    }
    return acc;
}, []);
export class Opponent extends utils.Actor {
    constructor(entity) {
        super(entity);
        this.onHurtBinding = this.onHurt.bind(this);
        entity.addTag("ldz_240913:distance_3");
        world.afterEvents.entityHurt.subscribe(this.onHurtBinding, { entities: [this.UnsafeEntity] });
    }
    get SoundPlayer() {
        return this.getActorComponent(OpponentSoundPlayer);
    }
    onConstructed() {
        this.addActorComponent(OpponentSoundPlayer, OPPONENT_SOUNDS);
        if (this.getProperty("ldz_240913:skin") === -1) {
            this.setProperty("ldz_240913:skin", Math.floor(Math.random() * SKINS_COUNT));
        }
        if (this.getProperty('ldz_240913:weapon') === "none") {
            const random = Math.random() * cumulativeWeights[cumulativeWeights.length - 1];
            const i = cumulativeWeights.findIndex(cumulativeWeight => random < cumulativeWeight);
            this.setProperty('ldz_240913:weapon', WEAPONS[i]);
            this.removeActorComponent(OpponentAttack);
            this.addActorComponent(OpponentAttack, ATTACKS[i]);
        }
        else {
            const i = WEAPONS.indexOf(this.getProperty('ldz_240913:weapon'));
            utils.mclog(`Opponent weapon set to ${this.getProperty('ldz_240913:weapon')} using attack at index ${i}`, LOGTAG);
            this.addActorComponent(OpponentAttack, ATTACKS[i]);
        }
        utils.mclog("Opponent constructed", LOGTAG);
    }
    destructor() {
        super.destructor();
    }
    attemptTaunt() {
        let entityCheck = false;
        let opponents = utils.getEntitiesSafe(this.dimension, { type: "ldz_240913:opponent", location: this.location, maxDistance: 5 });
        let entities = utils.getEntitiesSafe(this.dimension, { type: "player", location: this.location, maxDistance: 8, excludeGameModes: [GameMode.creative, GameMode.spectator] });
        entities.push(...utils.getEntitiesSafe(this.dimension, { families: ["ldz_240913:bruce_lee"], location: this.location, maxDistance: 8 }));
        if (entities.length > 0) {
            entities.forEach(entity => {
                var _a;
                if (entity.typeId === "ldz_240913:bruce_lee" && entity.getProperty('ldz_240913:health') !== 0) {
                    entityCheck = true;
                }
                if (entity.typeId === "minecraft:player" && ((_a = entity.Health) === null || _a === void 0 ? void 0 : _a.currentValue) !== 0) {
                    entityCheck = true;
                }
            });
        }
        if (entityCheck) {
            utils.mclog(`§uOpponent taunt skipped`, LOGTAG);
            return;
        }
        opponents.forEach(opponent => {
            if (!opponent.hasTag('taunting')) {
                opponent.addTag('taunting');
                this.taunt(opponent);
            }
        });
    }
    taunt(entity) {
        let taunt_delay = Math.random() * (TAUNT_DELAY_RANGE[1] - TAUNT_DELAY_RANGE[0]) + TAUNT_DELAY_RANGE[0];
        utils.waitForEvent(taunt_delay * 20, () => {
            var _a;
            let taunt = utils.getRandomInteger(TAUNT_RNG_RANGE[0], TAUNT_RNG_RANGE[1]);
            utils.mclog(`§uOpponent taunt ${taunt}`, LOGTAG);
            entity.addEffect("slowness", 100, { showParticles: false, amplifier: 255 });
            entity.playAnimation(`animation.ldz_240913.opponent.start_taunt_${taunt}`);
            (_a = this.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound("taunt");
        });
        utils.waitForEvent(taunt_delay, () => {
            entity.removeTag('taunting');
        });
    }
    onHurt(event) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        (_a = this.SoundPlayer) === null || _a === void 0 ? void 0 : _a.playSound("hurt");
        if (((_c = (_b = this.Health) === null || _b === void 0 ? void 0 : _b.currentValue) !== null && _c !== void 0 ? _c : 0) <= 0) {
            (_d = this.SoundPlayer) === null || _d === void 0 ? void 0 : _d.playSound("die");
            let dead = this.dimension.spawnEntity("ldz_240913:dead_opponent", this.location);
            dead.setProperty("ldz_240913:skin", (_e = this.getProperty("ldz_240913:skin")) !== null && _e !== void 0 ? _e : 0);
            dead.setProperty("ldz_240913:weapon", (_f = this.getProperty("ldz_240913:weapon")) !== null && _f !== void 0 ? _f : "unarmed");
            dead.teleport(this.location, { rotation: this.getRotation() });
            dead.applyKnockback((_h = (_g = event.damageSource.damagingEntity) === null || _g === void 0 ? void 0 : _g.getRotation().x) !== null && _h !== void 0 ? _h : 0, (_k = (_j = event.damageSource.damagingEntity) === null || _j === void 0 ? void 0 : _j.getRotation().y) !== null && _k !== void 0 ? _k : 0, 0.6, 0.3);
            utils.waitForEvent(1, () => dead.applyDamage(0.1, { cause: EntityDamageCause.selfDestruct }));
            utils.waitForEvent(15, () => {
                this.teleport(dead.location, { rotation: dead.getRotation() });
            });
        }
    }
}
Opponent.makeActors({ type: "ldz_240913:opponent" }, Opponent);
