import { world } from "@minecraft/server";
import { BruceLee } from "./bruce_lee/bruce_lee";
import { SafetyCheckedPlayer, getAllPlayersSafe, waitForTicks } from "./utils/index";
const outlineItems = [
    { typeId: "ldz_240913:guide_book", hud_only: false, distance: 20 },
    { typeId: "ldz_240913:dragon_crystal_shard", hud_only: true, distance: 5 },
    { typeId: "ldz_240913:scroll", hud_only: true, distance: 5 }
];
export class Outline {
    constructor(owner) {
        this.target = null;
        this.owner = owner;
        Outline.Outlines.push(this);
        this.update();
    }
    destructor() {
        Outline.Outlines = Outline.Outlines.filter(o => o !== this);
    }
    static initializeOutline() {
        world.afterEvents.playerSpawn.subscribe(event => {
            if (event.initialSpawn) {
                new Outline(new SafetyCheckedPlayer(event.player));
            }
        });
        getAllPlayersSafe().forEach(player => {
            new Outline(player);
        });
    }
    async update() {
        var _a, _b;
        while (this.owner.isValid()) {
            let foundTarget = false;
            let heldItem = (_a = this.owner.MainHand) === null || _a === void 0 ? void 0 : _a.typeId;
            outlineItems.forEach(item => {
                var _a, _b, _c, _d;
                if (heldItem === item.typeId) {
                    let targets = this.getEntitiesFromView(item.distance);
                    const actors = targets
                        .map(e => BruceLee.getActor(e.id, BruceLee))
                        .filter(e => {
                        if (e instanceof BruceLee) {
                            return e.Properties.OwnerId === this.owner.id;
                        }
                    });
                    let closest = actors[0];
                    if (closest && ((_a = this.target) === null || _a === void 0 ? void 0 : _a.id) !== closest.id) {
                        (_b = this.target) === null || _b === void 0 ? void 0 : _b.toggleOutline(false, this.owner);
                        this.target = closest;
                    }
                    if (closest) {
                        (_c = this.target) === null || _c === void 0 ? void 0 : _c.toggleOutline(!item.hud_only, this.owner);
                        if (item.hud_only) {
                            (_d = this.target) === null || _d === void 0 ? void 0 : _d.playAnimation("animation.ldz_240913.bruce_lee_hud.show", { players: [this.owner.name] });
                        }
                        foundTarget = true;
                    }
                }
            });
            if (!foundTarget) {
                (_b = this.target) === null || _b === void 0 ? void 0 : _b.toggleOutline(false, this.owner);
                this.target = null;
            }
            BruceLee.getActors(BruceLee).forEach(actor => {
                if (actor.isValid() && actor.Properties.OwnerId === this.owner.id && this.owner.hasTag("ldz_240913:show_panel")) {
                    actor.playAnimation("animation.ldz_240913.bruce_lee_hud.show", { players: [this.owner.name] });
                }
            });
            await waitForTicks(5);
        }
        this.destructor();
    }
    getEntitiesFromView(maxDistance) {
        const origin = this.owner.getHeadLocation();
        const families = ["ldz_240913:bruce_lee"];
        const includePassableBlocks = false;
        const options = { maxDistance, families, includePassableBlocks };
        const out = [];
        for (let index = 0; index < 5; index++) {
            out.push(...this.owner.dimension.getEntitiesFromRay(origin, this.owner.getViewDirection().rotateYaw(index, this.owner.RelativeUp), options)
                .map(result => result.entity));
            out.push(...this.owner.dimension.getEntitiesFromRay(origin, this.owner.getViewDirection().rotateYaw(-index, this.owner.RelativeUp), options)
                .map(result => result.entity));
        }
        return out;
    }
    static getTarget(player) {
        var _a, _b;
        return (_b = (_a = Outline.Outlines.find(o => o.owner.id === player.id)) === null || _a === void 0 ? void 0 : _a.target) !== null && _b !== void 0 ? _b : null;
    }
}
Outline.Outlines = [];
Outline.initializeOutline();
