import * as mc from "@minecraft/server";
import * as utils from "../utils/index";
const QUOTE_LIST = [
    "actually_i_do_not_teach",
    "all_type_of_knowledge",
    "combat",
    "empty_your_mind",
    "here_is_the_natural",
    "kung_fu_can_be_practiced",
    "kung_fu_is_originated",
    "magazine",
    "this_is_what_we_mean",
    "martial_art_has",
    "martial_art_include",
    "my_last_name_is_lee",
    "running_water",
    "steve",
    "styles_tends_to",
    "the_karate_punch",
    "the_word_superstar",
    "there_are_various_kind",
    "ultimately",
    "water_is_the_softest",
    "you_have_to_train",
    "you_know_what_i",
];
const quotesPropertyName = "quotes";
export function initializeScrollInteraction() {
    mc.system.afterEvents.scriptEventReceive.subscribe(event => {
        // Get player as safe
        if (!event.sourceEntity)
            return;
        const playerEntity = new utils.SafetyCheckedEntity(event.sourceEntity).getPlayer();
        if (!playerEntity)
            return;
        const player = new utils.SafetyCheckedPlayer(playerEntity);
        // Get current quotes (if any)
        let quotes = getQuotes(player);
        // Read the quotes from a player's list
        if (event.id === "ldz_240913:read_quotes" && event.sourceEntity) {
            if (!quotes)
                return;
            quotes.forEach(quote => { utils.mclog(quote); });
        }
        // Clear the quotes from a player's list
        if (event.id === "ldz_240913:clear_quotes" && event.sourceEntity) {
            utils.clearDynamicProperty(player, quotesPropertyName);
        }
        // Spawn a scroll with a specific quote attached
        if (event.id === "ldz_240913:spawn_scroll" && event.sourceEntity) {
            const scroll = utils.spawnEntitySafe(player.dimension, "ldz_240913:scroll", player.location);
            setScrollQuote(scroll, event.message);
        }
    });
    utils.afterEvents.onInteractWithEntity.subscribe(event => {
        if (event.victim.typeId === "ldz_240913:scroll") {
            // Get player as safe
            if (!event.player)
                return;
            const player = new utils.SafetyCheckedPlayer(event.player);
            if (!player)
                return;
            let selectedQuote = getScrollQuote(event.victim);
            if (!selectedQuote) {
                selectedQuote = getNewQuote(player);
                setScrollQuote(event.victim, selectedQuote);
                utils.mclog("Setting up new quote for scroll");
            }
            const menu = new utils.MenuBuilder(getQuoteSingleMenu(player, selectedQuote, new utils.SafetyCheckedEntity(event.victim)));
            menu.showMenuToPlayer(player).then(result => { if (result === utils.MenuResult.UserClosed)
                stopSound(player, getSound(selectedQuote)); });
        }
    });
}
function getQuoteSingleMenu(player, selectedQuote, scrollEntity) {
    return {
        type: "buttons",
        buttonLabel: { translate: getLang(selectedQuote), with: ["\n"] },
        title: { translate: "ldz_240913:menu.quote_single.title", with: ["\n"] },
        icon: "textures/ldz/240913/items/scroll",
        body: { rawtext: [{ translate: "ldz_240913:menu.quote_single.prefix" }, { text: "\n\n§o" }, { translate: getLang(selectedQuote), with: ["\n"] }, { text: "\n\n" }] },
        backButtonPosition: scrollEntity !== undefined ? "none" : "bottom",
        onPageOpen(player) {
            stopAllQuoteSounds(player);
            player.runCommand(`playsound ${getSound(selectedQuote)} @s`);
        },
        contents: [
            {
                text: { translate: "ldz_240913:menu.quote_single.button" },
                texture: "textures/ldz/240913/items/scroll",
                shouldHide: (player) => {
                    return scrollEntity === undefined;
                },
                onClick: (player) => {
                    if (scrollEntity) {
                        scrollEntity.triggerEvent("roll_up");
                        utils.waitForEvent(9, () => {
                            if (player.isValid()) {
                                let scroll = new mc.ItemStack("ldz_240913:scroll");
                                scroll.setLore(["+50 experience"]);
                                player.dimension.spawnItem(scroll, player.location);
                                // player.runCommand(`loot give @s loot "ldz/240913/scroll"`);
                                player.runCommand(`playsound random.pop @s ~ ~ ~ 0.35 1.2`);
                            }
                        });
                    }
                }
            }
        ]
    };
}
export function generateQuoteButtons(player) {
    const heardQuotes = getQuotes(player);
    let buttons = {};
    QUOTE_LIST.forEach(quote => {
        if (heardQuotes === null || heardQuotes === void 0 ? void 0 : heardQuotes.includes(quote)) {
            buttons[quote] = getQuoteSingleMenu(player, quote);
        }
        else { // push a blank button
            buttons[quote] = {
                type: "buttons",
                title: { translate: "ldz_240913:menu.quote_list.button_hidden" },
                body: { rawtext: [{ translate: "ldz_240913:menu.quotes.hidden" }, { text: "\n " }] },
                contents: [],
                icon: "textures/ldz/240913/items/scroll_missing"
            };
        }
    });
    return buttons;
}
function getQuotes(player) {
    let quotes = utils.readDynamicProperty(player, quotesPropertyName);
    if (!quotes)
        return undefined;
    else
        return quotes;
}
function getNewQuote(player) {
    let quotes = getQuotes(player);
    let chosenQuote = "";
    utils.mclog("Previous quotes seen: " + (quotes === null || quotes === void 0 ? void 0 : quotes.length));
    if (!quotes) {
        chosenQuote = utils.getRandomArrayEntry(QUOTE_LIST);
        utils.mclog("Never seen a quote before");
    }
    if (quotes && quotes.length < QUOTE_LIST.length) {
        const unheardQuotes = QUOTE_LIST.filter(quote => !(quotes === null || quotes === void 0 ? void 0 : quotes.includes(quote)));
        chosenQuote = utils.getRandomArrayEntry(unheardQuotes);
        utils.mclog("Adding a new quote");
    }
    if (chosenQuote !== "") {
        addQuote(player, chosenQuote);
    }
    if (quotes && quotes.length === QUOTE_LIST.length) {
        chosenQuote = utils.getRandomArrayEntry(QUOTE_LIST);
        utils.mclog("Already seen all quotes. Picking a random one");
    }
    return chosenQuote;
}
function addQuote(player, quote) {
    let quotes = utils.readDynamicProperty(player, quotesPropertyName);
    if (!quotes) {
        utils.writeDynamicProperty(player, quotesPropertyName, [quote]);
    }
    if (quotes) {
        quotes.push(quote);
        utils.writeDynamicProperty(player, quotesPropertyName, quotes);
    }
}
function setScrollQuote(scroll, quote) {
    utils.writeDynamicProperty(scroll, quotesPropertyName, [quote]);
}
function getScrollQuote(scroll) {
    let quote = utils.readDynamicProperty(scroll, quotesPropertyName);
    return quote;
}
export function getLang(key) {
    return `ldz_240913:bruce_lee.quote.${key}`;
}
export function getSound(key) {
    return `ldz_240913:bruce_lee.quote.${key}`;
}
export function stopSound(player, sound) {
    player.runCommand(`stopsound @s ${sound}`);
}
export function stopAllQuoteSounds(player) {
    QUOTE_LIST.forEach(quote => { stopSound(player, getSound(quote)); });
}
